import tgram

from typing import Union, List, Optional, Literal

from io import BytesIO
from pathlib import Path


MEDIA_TYPES = {
    "audio",
    "video",
    "photo",
    "animation",
    "voice",
    "video_note",
    "sticker",
    "document",
}

SERVICE_TYPES = {
    "video_chat_participants_invited",
    "video_chat_ended",
    "video_chat_started",
    "video_chat_scheduled",
    "giveaway_completed",
    "general_forum_topic_unhidden",
    "general_forum_topic_hidden",
    "forum_topic_reopened",
    "forum_topic_closed",
    "forum_topic_edited",
    "forum_topic_created",
    "chat_background_set",
    "checklist_tasks_added",
    "checklist_tasks_done",
    "direct_message_price_changed",
    "boost_added",
    "proximity_alert_triggered",
    "write_access_allowed",
    "successful_payment",
    "refunded_payment",
    "users_shared",
    "chat_shared",
    "pinned_message",
    "message_auto_delete_timer_changed",
    "channel_chat_created",
    "supergroup_chat_created",
    "group_chat_created",
    "delete_chat_photo",
    "new_chat_photo",
    "new_chat_title",
    "left_chat_member",
    "new_chat_members",
}


class MessageB:
    async def reply_text(
        self: "tgram.types.Message",
        text: str,
        message_thread_id: int = None,
        parse_mode: "tgram.types.ParseMode" = None,
        entities: List["tgram.types.MessageEntity"] = None,
        link_preview_options: "tgram.types.LinkPreviewOptions" = None,
        disable_notification: bool = None,
        protect_content: bool = None,
        message_effect_id: str = None,
        reply_markup: Union[
            "tgram.types.InlineKeyboardMarkup",
            "tgram.types.ReplyKeyboardMarkup",
            "tgram.types.ReplyKeyboardRemove",
            "tgram.types.ForceReply",
        ] = None,
    ) -> "tgram.types.Message":
        """
        Sends a text message in reply to the current message.

        Args:
            text (str): The text of the message to be sent.
            message_thread_id (int, optional): Unique identifier for the target message thread (topic) of the forum; for forum supergroups only.
            parse_mode (tgram.types.ParseMode, optional): Mode for parsing entities in the message text.
            entities (List[tgram.types.MessageEntity], optional): List of special entities that appear in message text, which can be specified instead of parse_mode.
            link_preview_options (tgram.types.LinkPreviewOptions, optional): Options for link previews.
            disable_notification (bool, optional): Sends the message silently. Users will receive a notification with no sound.
            protect_content (bool, optional): Protects the contents of the sent message from forwarding and saving.
            message_effect_id (str, optional): Identifier of the message effect.
            reply_markup (Union[tgram.types.InlineKeyboardMarkup, tgram.types.ReplyKeyboardMarkup, tgram.types.ReplyKeyboardRemove, tgram.types.ForceReply], optional): Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove reply keyboard or to force a reply from the user.

        Returns:
            tgram.types.Message: The sent message.

        Example:
            ```python
            await message.reply_text(
                text="<code>Hello, world!</code>",
                parse_mode="HTML",
                disable_notification=True
            )
            ```
        """
        return await self._me.send_message(
            chat_id=self.chat.id,
            text=text,
            business_connection_id=self.business_connection_id,
            message_thread_id=message_thread_id,
            parse_mode=parse_mode,
            entities=entities,
            link_preview_options=link_preview_options or self._me.link_preview_options,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self._me.protect_content,
            message_effect_id=message_effect_id,
            reply_markup=reply_markup,
            reply_parameters=self.__reply_param,
        )

    reply = reply_text

    async def reply_photo(
        self: "tgram.types.Message",
        photo: Union[Path, bytes, str],
        caption: str = None,
        message_thread_id: int = None,
        parse_mode: "tgram.types.ParseMode" = None,
        caption_entities: List["tgram.types.MessageEntity"] = None,
        show_caption_above_media: bool = None,
        has_spoiler: bool = None,
        disable_notification: bool = None,
        protect_content: bool = None,
        message_effect_id: str = None,
        reply_markup: Union[
            "tgram.types.InlineKeyboardMarkup",
            "tgram.types.ReplyKeyboardMarkup",
            "tgram.types.ReplyKeyboardRemove",
            "tgram.types.ForceReply",
        ] = None,
    ) -> "tgram.types.Message":
        """
        Sends a photo in reply to the current message.

        Args:
            photo (Union[Path, bytes, str]): The photo to be sent. Can be a file path, bytes, or a URL.
            caption (str, optional): Caption for the photo.
            message_thread_id (int, optional): Unique identifier for the target message thread (topic) of the forum; for forum supergroups only.
            parse_mode (tgram.types.ParseMode, optional): Mode for parsing entities in the photo caption.
            caption_entities (List[tgram.types.MessageEntity], optional): List of special entities that appear in the caption, which can be specified instead of parse_mode.
            show_caption_above_media (bool, optional): Whether to show the caption above the media.
            has_spoiler (bool, optional): Whether the photo contains a spoiler.
            disable_notification (bool, optional): Sends the message silently. Users will receive a notification with no sound.
            protect_content (bool, optional): Protects the contents of the sent message from forwarding and saving.
            message_effect_id (str, optional): Identifier of the message effect.
            reply_markup (Union[tgram.types.InlineKeyboardMarkup, tgram.types.ReplyKeyboardMarkup, tgram.types.ReplyKeyboardRemove, tgram.types.ForceReply], optional): Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove reply keyboard or to force a reply from the user.

        Returns:
            tgram.types.Message: The sent message.

        Example:
            ```python
            await message.reply_photo(
                photo="path/to/photo.jpg",
                caption="Check out this *photo!*",
                parse_mode="Markdown",
                disable_notification=True
            )
            ```
        """
        return await self._me.send_photo(
            self.chat.id,
            photo=photo,
            business_connection_id=self.business_connection_id,
            message_thread_id=message_thread_id,
            caption=caption,
            parse_mode=parse_mode,
            caption_entities=caption_entities,
            show_caption_above_media=show_caption_above_media,
            has_spoiler=has_spoiler,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self._me.protect_content,
            message_effect_id=message_effect_id,
            reply_parameters=self.__reply_param,
            reply_markup=reply_markup,
        )

    async def reply_audio(
        self: "tgram.types.Message",
        audio: Union[Path, bytes, str],
        caption: str = None,
        message_thread_id: int = None,
        parse_mode: "tgram.types.ParseMode" = None,
        caption_entities: List["tgram.types.MessageEntity"] = None,
        duration: int = None,
        performer: str = None,
        title: str = None,
        thumbnail: Union[Path, bytes, str] = None,
        disable_notification: bool = None,
        protect_content: bool = None,
        message_effect_id: str = None,
        reply_markup: Union[
            "tgram.types.InlineKeyboardMarkup",
            "tgram.types.ReplyKeyboardMarkup",
            "tgram.types.ReplyKeyboardRemove",
            "tgram.types.ForceReply",
        ] = None,
    ) -> "tgram.types.Message":
        """
        Sends an audio file in reply to the current message.

        Args:
            audio (Union[Path, bytes, str]): The audio file to be sent. Can be a file path, bytes, or a URL.
            caption (str, optional): Caption for the audio file.
            message_thread_id (int, optional): Unique identifier for the target message thread (topic) of the forum; for forum supergroups only.
            parse_mode (tgram.types.ParseMode, optional): Mode for parsing entities in the audio caption.
            caption_entities (List[tgram.types.MessageEntity], optional): List of special entities that appear in the caption, which can be specified instead of parse_mode.
            duration (int, optional): Duration of the audio in seconds.
            performer (str, optional): Performer of the audio.
            title (str, optional): Title of the audio.
            thumbnail (Union[Path, bytes, str], optional): Thumbnail of the audio file.
            disable_notification (bool, optional): Sends the message silently. Users will receive a notification with no sound.
            protect_content (bool, optional): Protects the contents of the sent message from forwarding and saving.
            message_effect_id (str, optional): Identifier of the message effect.
            reply_markup (Union[tgram.types.InlineKeyboardMarkup, tgram.types.ReplyKeyboardMarkup, tgram.types.ReplyKeyboardRemove, tgram.types.ForceReply], optional): Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove reply keyboard or to force a reply from the user.

        Returns:
            tgram.types.Message: The sent message.

        Example:
            ```python
            await message.reply_audio(
                audio="path/to/audio.mp3",
                caption="Here is the audio file.",
                parse_mode="Markdown",
                duration=120,
                performer="Artist Name",
                title="Audio Title",
                disable_notification=True
            )
            ```
        """
        return await self._me.send_audio(
            self.chat.id,
            audio=audio,
            business_connection_id=self.business_connection_id,
            message_thread_id=message_thread_id,
            caption=caption,
            parse_mode=parse_mode,
            caption_entities=caption_entities,
            duration=duration,
            performer=performer,
            title=title,
            thumbnail=thumbnail,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self._me.protect_content,
            message_effect_id=message_effect_id,
            reply_parameters=self.__reply_param,
            reply_markup=reply_markup,
        )

    async def reply_document(
        self: "tgram.types.Message",
        document: Union[Path, bytes, str],
        message_thread_id: int = None,
        thumbnail: Union[Path, bytes, str] = None,
        caption: str = None,
        parse_mode: "tgram.types.ParseMode" = None,
        caption_entities: List["tgram.types.MessageEntity"] = None,
        disable_content_type_detection: bool = None,
        disable_notification: bool = None,
        protect_content: bool = None,
        message_effect_id: str = None,
        reply_markup: Union[
            "tgram.types.InlineKeyboardMarkup",
            "tgram.types.ReplyKeyboardMarkup",
            "tgram.types.ReplyKeyboardRemove",
            "tgram.types.ForceReply",
        ] = None,
    ) -> "tgram.types.Message":
        """
        Sends a document in reply to the current message.

        Args:
            document (Union[Path, bytes, str]): The document to send. Can be a file path, bytes, or a URL.
            message_thread_id (int, optional): Unique identifier for the target message thread (topic) of the forum; for forum supergroups only.
            thumbnail (Union[Path, bytes, str], optional): Thumbnail of the file sent; can be ignored if server-side thumbnail generation is supported.
            caption (str, optional): Document caption (may also be used when resending documents by file_id), 0-1024 characters after entities parsing.
            parse_mode (tgram.types.ParseMode, optional): Mode for parsing entities in the document caption.
            caption_entities (List[tgram.types.MessageEntity], optional): List of special entities that appear in the caption, which can be specified instead of parse_mode.
            disable_content_type_detection (bool, optional): Disables automatic server-side content type detection for files uploaded using multipart/form-data.
            disable_notification (bool, optional): Sends the message silently. Users will receive a notification with no sound.
            protect_content (bool, optional): Protects the contents of the sent message from forwarding and saving.
            message_effect_id (str, optional): Unique identifier for the message effect.
            reply_markup (Union[tgram.types.InlineKeyboardMarkup, tgram.types.ReplyKeyboardMarkup, tgram.types.ReplyKeyboardRemove, tgram.types.ForceReply], optional): Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove reply keyboard or to force a reply from the user.

        Returns:
            tgram.types.Message: The sent message.

        Example:
            ```python
            await message.reply_document(
                document="path/to/document.pdf",
                caption="Here is the document you requested.",
                parse_mode="Markdown",
                disable_notification=True
            )
            ```
        """
        return await self._me.send_document(
            self.chat.id,
            document=document,
            business_connection_id=self.business_connection_id,
            message_thread_id=message_thread_id,
            thumbnail=thumbnail,
            caption=caption,
            parse_mode=parse_mode,
            caption_entities=caption_entities,
            disable_content_type_detection=disable_content_type_detection,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self._me.protect_content,
            message_effect_id=message_effect_id,
            reply_parameters=self.__reply_param,
            reply_markup=reply_markup,
        )

    async def reply_video(
        self: "tgram.types.Message",
        video: Union[Path, bytes, str],
        message_thread_id: int = None,
        duration: int = None,
        width: int = None,
        height: int = None,
        thumbnail: Union[Path, bytes, str] = None,
        caption: str = None,
        parse_mode: "tgram.types.ParseMode" = None,
        caption_entities: List["tgram.types.MessageEntity"] = None,
        show_caption_above_media: bool = None,
        has_spoiler: bool = None,
        supports_streaming: bool = None,
        disable_notification: bool = None,
        protect_content: bool = None,
        message_effect_id: str = None,
        reply_markup: Union[
            "tgram.types.InlineKeyboardMarkup",
            "tgram.types.ReplyKeyboardMarkup",
            "tgram.types.ReplyKeyboardRemove",
            "tgram.types.ForceReply",
        ] = None,
    ) -> "tgram.types.Message":
        """
        Sends a video in reply to the current message.

        Args:
            video (Union[Path, bytes, str]): The video to be sent. Can be a file path, bytes, or a URL.
            message_thread_id (int, optional): Unique identifier for the target message thread (topic) of the forum; for forum supergroups only.
            duration (int, optional): Duration of the video in seconds.
            width (int, optional): Video width.
            height (int, optional): Video height.
            thumbnail (Union[Path, bytes, str], optional): Thumbnail of the video.
            caption (str, optional): Caption for the video.
            parse_mode (tgram.types.ParseMode, optional): Mode for parsing entities in the video caption.
            caption_entities (List[tgram.types.MessageEntity], optional): List of special entities that appear in the caption, which can be specified instead of parse_mode.
            show_caption_above_media (bool, optional): Whether to show the caption above the media.
            has_spoiler (bool, optional): Whether the video contains a spoiler.
            supports_streaming (bool, optional): Whether the video supports streaming.
            disable_notification (bool, optional): Sends the message silently. Users will receive a notification with no sound.
            protect_content (bool, optional): Protects the contents of the sent message from forwarding and saving.
            message_effect_id (str, optional): Identifier of the message effect.
            reply_markup (Union[tgram.types.InlineKeyboardMarkup, tgram.types.ReplyKeyboardMarkup, tgram.types.ReplyKeyboardRemove, tgram.types.ForceReply], optional): Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove reply keyboard or to force a reply from the user.

        Returns:
            tgram.types.Message: The sent message.

        Example:
            ```python
            await message.reply_video(
                video="path/to/video.mp4",
                caption="Check out this video!",
                parse_mode="Markdown",
                disable_notification=True
            )
            ```
        """
        return await self._me.send_video(
            self.chat.id,
            video=video,
            business_connection_id=self.business_connection_id,
            message_thread_id=message_thread_id,
            duration=duration,
            width=width,
            height=height,
            thumbnail=thumbnail,
            caption=caption,
            parse_mode=parse_mode,
            caption_entities=caption_entities,
            show_caption_above_media=show_caption_above_media,
            has_spoiler=has_spoiler,
            supports_streaming=supports_streaming,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self._me.protect_content,
            message_effect_id=message_effect_id,
            reply_parameters=self.__reply_param,
            reply_markup=reply_markup,
        )

    async def reply_animation(
        self: "tgram.types.Message",
        animation: Union[Path, bytes, str],
        message_thread_id: int = None,
        duration: int = None,
        width: int = None,
        height: int = None,
        thumbnail: Union[Path, bytes, str] = None,
        caption: str = None,
        parse_mode: "tgram.types.ParseMode" = None,
        caption_entities: List["tgram.types.MessageEntity"] = None,
        show_caption_above_media: bool = None,
        has_spoiler: bool = None,
        disable_notification: bool = None,
        protect_content: bool = None,
        message_effect_id: str = None,
        reply_markup: Union[
            "tgram.types.InlineKeyboardMarkup",
            "tgram.types.ReplyKeyboardMarkup",
            "tgram.types.ReplyKeyboardRemove",
            "tgram.types.ForceReply",
        ] = None,
    ) -> "tgram.types.Message":
        """
        Sends an animation (GIF) in reply to the current message.

        Args:
            animation (Union[Path, bytes, str]): The animation to be sent. Can be a file path, bytes, or a URL.
            message_thread_id (int, optional): Unique identifier for the target message thread (topic) of the forum; for forum supergroups only.
            duration (int, optional): Duration of the animation in seconds.
            width (int, optional): Width of the animation.
            height (int, optional): Height of the animation.
            thumbnail (Union[Path, bytes, str], optional): Thumbnail of the animation.
            caption (str, optional): Caption for the animation.
            parse_mode (tgram.types.ParseMode, optional): Mode for parsing entities in the animation caption.
            caption_entities (List[tgram.types.MessageEntity], optional): List of special entities that appear in the caption, which can be specified instead of parse_mode.
            show_caption_above_media (bool, optional): Whether to show the caption above the media.
            has_spoiler (bool, optional): Whether the animation contains a spoiler.
            disable_notification (bool, optional): Sends the message silently. Users will receive a notification with no sound.
            protect_content (bool, optional): Protects the contents of the sent message from forwarding and saving.
            message_effect_id (str, optional): Identifier of the message effect.
            reply_markup (Union[tgram.types.InlineKeyboardMarkup, tgram.types.ReplyKeyboardMarkup, tgram.types.ReplyKeyboardRemove, tgram.types.ForceReply], optional): Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove reply keyboard or to force a reply from the user.

        Returns:
            tgram.types.Message: The sent message.

        Example:
            ```python
            await message.reply_animation(
                animation="path/to/animation.gif",
                caption="Check out this cool animation!",
                parse_mode="Markdown",
                disable_notification=True
            )
            ```
        """
        return await self._me.send_animation(
            self.chat.id,
            animation=animation,
            business_connection_id=self.business_connection_id,
            message_thread_id=message_thread_id,
            duration=duration,
            width=width,
            height=height,
            thumbnail=thumbnail,
            caption=caption,
            parse_mode=parse_mode,
            caption_entities=caption_entities,
            show_caption_above_media=show_caption_above_media,
            has_spoiler=has_spoiler,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self._me.protect_content,
            message_effect_id=message_effect_id,
            reply_parameters=self.__reply_param,
            reply_markup=reply_markup,
        )

    async def reply_voice(
        self: "tgram.types.Message",
        voice: Union[Path, bytes, str],
        caption: str = None,
        message_thread_id: int = None,
        parse_mode: "tgram.types.ParseMode" = None,
        caption_entities: List["tgram.types.MessageEntity"] = None,
        duration: int = None,
        disable_notification: bool = None,
        protect_content: bool = None,
        message_effect_id: str = None,
        reply_markup: Union[
            "tgram.types.InlineKeyboardMarkup",
            "tgram.types.ReplyKeyboardMarkup",
            "tgram.types.ReplyKeyboardRemove",
            "tgram.types.ForceReply",
        ] = None,
    ) -> "tgram.types.Message":
        """
        Sends a voice message in reply to the current message.

        Args:
            voice (Union[Path, bytes, str]): The voice message to be sent. Can be a file path, bytes, or a URL.
            caption (str, optional): Caption for the voice message.
            message_thread_id (int, optional): Unique identifier for the target message thread (topic) of the forum; for forum supergroups only.
            parse_mode (tgram.types.ParseMode, optional): Mode for parsing entities in the voice message caption.
            caption_entities (List[tgram.types.MessageEntity], optional): List of special entities that appear in the caption, which can be specified instead of parse_mode.
            duration (int, optional): Duration of the voice message in seconds.
            disable_notification (bool, optional): Sends the message silently. Users will receive a notification with no sound.
            protect_content (bool, optional): Protects the contents of the sent message from forwarding and saving.
            message_effect_id (str, optional): Identifier of the message effect.
            reply_markup (Union[tgram.types.InlineKeyboardMarkup, tgram.types.ReplyKeyboardMarkup, tgram.types.ReplyKeyboardRemove, tgram.types.ForceReply], optional): Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove reply keyboard or to force a reply from the user.

        Returns:
            tgram.types.Message: The sent message.

        Example:
            ```python
            await message.reply_voice(
                voice="path/to/voice.ogg",
                caption="Here is the voice message.",
                parse_mode="Markdown",
                duration=60,
                disable_notification=True
            )
            ```
        """
        return await self._me.send_voice(
            self.chat.id,
            voice=voice,
            business_connection_id=self.business_connection_id,
            message_thread_id=message_thread_id,
            caption=caption,
            parse_mode=parse_mode,
            caption_entities=caption_entities,
            duration=duration,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self._me.protect_content,
            message_effect_id=message_effect_id,
            reply_parameters=self.__reply_param,
            reply_markup=reply_markup,
        )

    async def reply_video_note(
        self: "tgram.types.Message",
        video_note: Union[Path, bytes, str],
        message_thread_id: int = None,
        duration: int = None,
        length: int = None,
        thumbnail: Union[Path, bytes, str] = None,
        disable_notification: bool = None,
        protect_content: bool = None,
        message_effect_id: str = None,
        reply_markup: Union[
            "tgram.types.InlineKeyboardMarkup",
            "tgram.types.ReplyKeyboardMarkup",
            "tgram.types.ReplyKeyboardRemove",
            "tgram.types.ForceReply",
        ] = None,
    ) -> "tgram.types.Message":
        """
        Sends a video note in reply to the current message.

        Args:
            video_note (Union[Path, bytes, str]): The video note to be sent. Can be a file path, bytes, or a URL.
            message_thread_id (int, optional): Unique identifier for the target message thread (topic) of the forum; for forum supergroups only.
            duration (int, optional): Duration of the video note in seconds.
            length (int, optional): Video width and height (diameter of the video message) in pixels.
            thumbnail (Union[Path, bytes, str], optional): Thumbnail of the video note.
            disable_notification (bool, optional): Sends the message silently. Users will receive a notification with no sound.
            protect_content (bool, optional): Protects the contents of the sent message from forwarding and saving.
            message_effect_id (str, optional): Identifier of the message effect.
            reply_markup (Union[tgram.types.InlineKeyboardMarkup, tgram.types.ReplyKeyboardMarkup, tgram.types.ReplyKeyboardRemove, tgram.types.ForceReply], optional): Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove reply keyboard or to force a reply from the user.

        Returns:
            tgram.types.Message: The sent message.

        Example:
            ```python
            await message.reply_video_note(
                video_note="path/to/video_note.mp4",
                duration=60,
                length=240,
                disable_notification=True
            )
            ```
        """
        return await self._me.send_video_note(
            self.chat.id,
            video_note=video_note,
            business_connection_id=self.business_connection_id,
            message_thread_id=message_thread_id,
            duration=duration,
            length=length,
            thumbnail=thumbnail,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self._me.protect_content,
            message_effect_id=message_effect_id,
            reply_parameters=self.__reply_param,
            reply_markup=reply_markup,
        )

    async def reply_media_group(
        self: "tgram.types.Message",
        media: List["tgram.types.InputMedia"],
        message_thread_id: int = None,
        disable_notification: bool = None,
        protect_content: bool = None,
        message_effect_id: str = None,
    ) -> List["tgram.types.Message"]:
        """
        Sends a group of photos or videos as an album in reply to the current message.

        Args:
            media (List[tgram.types.InputMedia]): A list of media to be sent. Each media item can be a photo or video.
            message_thread_id (int, optional): Unique identifier for the target message thread (topic) of the forum; for forum supergroups only.
            disable_notification (bool, optional): Sends the message silently. Users will receive a notification with no sound.
            protect_content (bool, optional): Protects the contents of the sent message from forwarding and saving.
            message_effect_id (str, optional): Identifier of the message effect.

        Returns:
            List[tgram.types.Message]: The sent messages.

        Example:
            ```python
            media = [
                tgram.types.InputMediaPhoto(media="path/to/photo1.jpg"),
                tgram.types.InputMediaPhoto(media="path/to/photo2.jpg"),
                tgram.types.InputMediaVideo(media="path/to/video.mp4")
            ]
            await message.reply_media_group(
                media=media,
                disable_notification=True
            )
            ```
        """
        return await self._me.send_media_group(
            self.chat.id,
            media=media,
            business_connection_id=self.business_connection_id,
            message_thread_id=message_thread_id,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self._me.protect_content,
            message_effect_id=message_effect_id,
            reply_parameters=self.__reply_param,
        )

    async def reply_location(
        self: "tgram.types.Message",
        latitude: float,
        longitude: float,
        message_thread_id: int = None,
        horizontal_accuracy: float = None,
        live_period: int = None,
        heading: int = None,
        proximity_alert_radius: int = None,
        disable_notification: bool = None,
        protect_content: bool = None,
        message_effect_id: str = None,
        reply_markup: Union[
            "tgram.types.InlineKeyboardMarkup",
            "tgram.types.ReplyKeyboardMarkup",
            "tgram.types.ReplyKeyboardRemove",
            "tgram.types.ForceReply",
        ] = None,
    ) -> "tgram.types.Message":
        """
        Sends a location in reply to the current message.

        Args:
            latitude (float): Latitude of the location.
            longitude (float): Longitude of the location.
            message_thread_id (int, optional): Unique identifier for the target message thread (topic) of the forum; for forum supergroups only.
            horizontal_accuracy (float, optional): The radius of uncertainty for the location, measured in meters; 0-1500.
            live_period (int, optional): Period in seconds for which the location will be updated (should be between 60 and 86400).
            heading (int, optional): For live locations, a direction in which the user is moving, in degrees; 1-360.
            proximity_alert_radius (int, optional): For live locations, a maximum distance for proximity alerts about approaching another chat member, in meters; 1-100000.
            disable_notification (bool, optional): Sends the message silently. Users will receive a notification with no sound.
            protect_content (bool, optional): Protects the contents of the sent message from forwarding and saving.
            message_effect_id (str, optional): Identifier of the message effect.
            reply_markup (Union[tgram.types.InlineKeyboardMarkup, tgram.types.ReplyKeyboardMarkup, tgram.types.ReplyKeyboardRemove, tgram.types.ForceReply], optional): Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove reply keyboard or to force a reply from the user.

        Returns:
            tgram.types.Message: The sent message.

        Example:
            ```python
            await message.reply_location(
                latitude=37.7749,
                longitude=-122.4194,
                live_period=3600,
                disable_notification=True
            )
            ```
        """
        return await self._me.send_location(
            self.chat.id,
            latitude=latitude,
            longitude=longitude,
            business_connection_id=self.business_connection_id,
            message_thread_id=message_thread_id,
            horizontal_accuracy=horizontal_accuracy,
            live_period=live_period,
            heading=heading,
            proximity_alert_radius=proximity_alert_radius,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self._me.protect_content,
            message_effect_id=message_effect_id,
            reply_parameters=self.__reply_param,
            reply_markup=reply_markup,
        )

    async def reply_paid_media(
        self: "tgram.types.Message",
        star_count: int,
        media: List["tgram.types.InputPaidMedia"],
        caption: str = None,
        parse_mode: "tgram.types.ParseMode" = None,
        caption_entities: List["tgram.types.MessageEntity"] = None,
        show_caption_above_media: bool = None,
        disable_notification: bool = None,
        protect_content: bool = None,
        reply_markup: Union[
            "tgram.types.InlineKeyboardMarkup",
            "tgram.types.ReplyKeyboardMarkup",
            "tgram.types.ReplyKeyboardRemove",
            "tgram.types.ForceReply",
        ] = None,
    ) -> "tgram.types.Message":
        """
        Sends a paid media message in reply to the current message.

        Args:
            star_count (int): Number of stars required to view the media.
            media (List[tgram.types.InputPaidMedia]): A list of paid media to be sent.
            caption (str, optional): Caption for the media.
            parse_mode (tgram.types.ParseMode, optional): Mode for parsing entities in the media caption.
            caption_entities (List[tgram.types.MessageEntity], optional): List of special entities that appear in the caption, which can be specified instead of parse_mode.
            show_caption_above_media (bool, optional): Whether to show the caption above the media.
            disable_notification (bool, optional): Sends the message silently. Users will receive a notification with no sound.
            protect_content (bool, optional): Protects the contents of the sent message from forwarding and saving.
            reply_markup (Union[tgram.types.InlineKeyboardMarkup, tgram.types.ReplyKeyboardMarkup, tgram.types.ReplyKeyboardRemove, tgram.types.ForceReply], optional): Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove reply keyboard or to force a reply from the user.

        Returns:
            tgram.types.Message: The sent message.

        Example:
            ```python
            media = [
                tgram.types.InputPaidMedia(media="path/to/photo1.jpg"),
                tgram.types.InputPaidMedia(media="path/to/photo2.jpg")
            ]
            await message.reply_paid_media(
                star_count=5,
                media=media,
                caption="Check out this paid media!",
                parse_mode="Markdown",
                disable_notification=True
            )
            ```
        """
        return await self._me.send_paid_media(
            self.chat.id,
            star_count=star_count,
            media=media,
            caption=caption,
            parse_mode=parse_mode or self.me_parse_mode,
            caption_entities=caption_entities,
            show_caption_above_media=show_caption_above_media,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self._me.protect_content,
            reply_parameters=self.__reply_param,
            reply_markup=reply_markup,
            business_connection_id=self.business_connection_id,
        )

    async def reply_chat_action(
        self: "tgram.types.Message", action: str, message_thread_id: int = None
    ) -> bool:
        """
        Sends a chat action in reply to the current message.

        Args:
            action (str): Type of action to broadcast. Choose one, depending on what the user is about to receive: 'typing' for text messages, 'upload_photo' for photos, 'record_video' or 'upload_video' for videos, 'record_voice' or 'upload_voice' for voice notes, 'upload_document' for general files, 'find_location' for location data, 'record_video_note' or 'upload_video_note' for video notes.
            message_thread_id (int, optional): Unique identifier for the target message thread (topic) of the forum; for forum supergroups only.

        Returns:
            bool: True on success.

        Example:
            ```python
            await message.reply_chat_action(
                action="typing"
            )
            ```
        """
        return await self._me.send_chat_action(
            self.chat.id,
            action=action,
            business_connection_id=self.business_connection_id,
            message_thread_id=message_thread_id,
        )

    async def reply_contact(
        self: "tgram.types.Message",
        phone_number: str,
        first_name: str,
        last_name: str = None,
        vcard: str = None,
        message_thread_id: int = None,
        disable_notification: bool = None,
        protect_content: bool = None,
        message_effect_id: str = None,
        reply_markup: Union[
            "tgram.types.InlineKeyboardMarkup",
            "tgram.types.ReplyKeyboardMarkup",
            "tgram.types.ReplyKeyboardRemove",
            "tgram.types.ForceReply",
        ] = None,
    ) -> "tgram.types.Message":
        """
        Sends a contact in reply to the current message.

        Args:
            phone_number (str): Contact's phone number.
            first_name (str): Contact's first name.
            last_name (str, optional): Contact's last name.
            vcard (str, optional): Additional data about the contact in the form of a vCard, 0-2048 bytes.
            message_thread_id (int, optional): Unique identifier for the target message thread (topic) of the forum; for forum supergroups only.
            disable_notification (bool, optional): Sends the message silently. Users will receive a notification with no sound.
            protect_content (bool, optional): Protects the contents of the sent message from forwarding and saving.
            message_effect_id (str, optional): Identifier of the message effect.
            reply_markup (Union[tgram.types.InlineKeyboardMarkup, tgram.types.ReplyKeyboardMarkup, tgram.types.ReplyKeyboardRemove, tgram.types.ForceReply], optional): Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove reply keyboard or to force a reply from the user.

        Returns:
            tgram.types.Message: The sent message.

        Example:
            ```python
            await message.reply_contact(
                phone_number="+123456789",
                first_name="John",
                last_name="Doe",
                disable_notification=True
            )
            ```
        """
        return await self._me.send_contact(
            self.chat.id,
            phone_number=phone_number,
            first_name=first_name,
            business_connection_id=self.business_connection_id,
            message_thread_id=message_thread_id,
            last_name=last_name,
            vcard=vcard,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self._me.protect_content,
            message_effect_id=message_effect_id,
            reply_parameters=self.__reply_param,
            reply_markup=reply_markup,
        )

    async def reply_dice(
        self: "tgram.types.Message",
        emoji: str = None,
        message_thread_id: int = None,
        disable_notification: bool = None,
        protect_content: bool = None,
        message_effect_id: str = None,
        reply_markup: Union[
            "tgram.types.InlineKeyboardMarkup",
            "tgram.types.ReplyKeyboardMarkup",
            "tgram.types.ReplyKeyboardRemove",
            "tgram.types.ForceReply",
        ] = None,
    ) -> "tgram.types.Message":
        """
        Sends a dice message in reply to the current message.

        Args:
            emoji (str, optional): Emoji on which the dice throw animation is based. Currently, must be one of '🎲', '🎯', '🏀', '⚽', '🎳', or '🎰'.
            message_thread_id (int, optional): Unique identifier for the target message thread (topic) of the forum; for forum supergroups only.
            disable_notification (bool, optional): Sends the message silently. Users will receive a notification with no sound.
            protect_content (bool, optional): Protects the contents of the sent message from forwarding and saving.
            message_effect_id (str, optional): Identifier of the message effect.
            reply_markup (Union[tgram.types.InlineKeyboardMarkup, tgram.types.ReplyKeyboardMarkup, tgram.types.ReplyKeyboardRemove, tgram.types.ForceReply], optional): Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove reply keyboard or to force a reply from the user.

        Returns:
            tgram.types.Message: The sent message.

        Example:
            ```python
            await message.reply_dice(
                emoji="🎲",
                disable_notification=True
            )
            ```
        """
        return await self._me.send_dice(
            self.chat.id,
            business_connection_id=self.business_connection_id,
            message_thread_id=message_thread_id,
            emoji=emoji,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self._me.protect_content,
            message_effect_id=message_effect_id,
            reply_parameters=self.__reply_param,
            reply_markup=reply_markup,
        )

    async def reply_game(
        self: "tgram.types.Message",
        game_short_name: str,
        message_thread_id: int = None,
        disable_notification: bool = None,
        protect_content: bool = None,
        message_effect_id: str = None,
        reply_markup: "tgram.types.InlineKeyboardMarkup" = None,
    ) -> "tgram.types.Message":
        """
        Sends a game in reply to the current message.

        Args:
            game_short_name (str): Short name of the game.
            message_thread_id (int, optional): Unique identifier for the target message thread (topic) of the forum; for forum supergroups only.
            disable_notification (bool, optional): Sends the message silently. Users will receive a notification with no sound.
            protect_content (bool, optional): Protects the contents of the sent message from forwarding and saving.
            message_effect_id (str, optional): Identifier of the message effect.
            reply_markup (tgram.types.InlineKeyboardMarkup, optional): Additional interface options. A JSON-serialized object for an inline keyboard.

        Returns:
            tgram.types.Message: The sent message.

        Example:
            ```python
            await message.reply_game(
                game_short_name="game_name",
                disable_notification=True
            )
            ```
        """
        return await self._me.send_game(
            self.chat.id,
            game_short_name=game_short_name,
            business_connection_id=self.business_connection_id,
            message_thread_id=message_thread_id,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self._me.protect_content,
            message_effect_id=message_effect_id,
            reply_parameters=self.__reply_param,
            reply_markup=reply_markup,
        )

    async def reply_invoice(
        self: "tgram.types.Message",
        title: str,
        description: str,
        payload: str,
        currency: str,
        prices: List["tgram.types.LabeledPrice"],
        provider_token: str = None,
        max_tip_amount: int = None,
        suggested_tip_amounts: List[int] = None,
        start_parameter: str = None,
        provider_data: str = None,
        photo_url: str = None,
        photo_size: int = None,
        photo_width: int = None,
        photo_height: int = None,
        need_name: bool = None,
        need_phone_number: bool = None,
        need_email: bool = None,
        need_shipping_address: bool = None,
        send_phone_number_to_provider: bool = None,
        send_email_to_provider: bool = None,
        is_flexible: bool = None,
        message_thread_id: int = None,
        disable_notification: bool = None,
        protect_content: bool = None,
        message_effect_id: str = None,
        reply_markup: "tgram.types.InlineKeyboardMarkup" = None,
    ) -> "tgram.types.Message":
        """
        Sends an invoice in reply to the current message.

        Args:
            title (str): Product name, 1-32 characters.
            description (str): Product description, 1-255 characters.
            payload (str): Bot-defined invoice payload, 1-128 bytes. This will not be displayed to the user, use for your internal processes.
            currency (str): Three-letter ISO 4217 currency code.
            prices (List[tgram.types.LabeledPrice]): Price breakdown, a list of components (e.g. product price, tax, discount, delivery cost, delivery tax, bonus, etc.).
            provider_token (str, optional): Payment provider token, obtained via BotFather.
            max_tip_amount (int, optional): The maximum accepted amount for tips in the smallest units of the currency.
            suggested_tip_amounts (List[int], optional): A list of suggested amounts of tips in the smallest units of the currency.
            start_parameter (str, optional): Unique deep-linking parameter that can be used to generate this invoice when used as a start parameter.
            provider_data (str, optional): JSON-encoded data about the invoice, which will be shared with the payment provider.
            photo_url (str, optional): URL of the product photo for the invoice.
            photo_size (int, optional): Photo size.
            photo_width (int, optional): Photo width.
            photo_height (int, optional): Photo height.
            need_name (bool, optional): Pass True if you require the user's full name to complete the order.
            need_phone_number (bool, optional): Pass True if you require the user's phone number to complete the order.
            need_email (bool, optional): Pass True if you require the user's email address to complete the order.
            need_shipping_address (bool, optional): Pass True if you require the user's shipping address to complete the order.
            send_phone_number_to_provider (bool, optional): Pass True if the user's phone number should be sent to the provider.
            send_email_to_provider (bool, optional): Pass True if the user's email address should be sent to the provider.
            is_flexible (bool, optional): Pass True if the final price depends on the shipping method.
            message_thread_id (int, optional): Unique identifier for the target message thread (topic) of the forum; for forum supergroups only.
            disable_notification (bool, optional): Sends the message silently. Users will receive a notification with no sound.
            protect_content (bool, optional): Protects the contents of the sent message from forwarding and saving.
            message_effect_id (str, optional): Identifier of the message effect.
            reply_markup (tgram.types.InlineKeyboardMarkup, optional): Additional interface options. A JSON-serialized object for an inline keyboard.

        Returns:
            tgram.types.Message: The sent message.

        Example:
            ```python
            prices = [tgram.types.LabeledPrice(label="Product", amount=1000)]
            await message.reply_invoice(
                title="Product Title",
                description="Product Description",
                payload="payload",
                currency="USD",
                prices=prices,
                provider_token="provider_token",
                disable_notification=True
            )
            ```
        """
        return await self._me.send_invoice(
            self.chat.id,
            title=title,
            description=description,
            payload=payload,
            currency=currency,
            prices=prices,
            message_thread_id=message_thread_id,
            provider_token=provider_token,
            max_tip_amount=max_tip_amount,
            suggested_tip_amounts=suggested_tip_amounts,
            start_parameter=start_parameter,
            provider_data=provider_data,
            photo_url=photo_url,
            photo_size=photo_size,
            photo_width=photo_width,
            photo_height=photo_height,
            need_name=need_name,
            need_phone_number=need_phone_number,
            need_email=need_email,
            need_shipping_address=need_shipping_address,
            send_phone_number_to_provider=send_phone_number_to_provider,
            send_email_to_provider=send_email_to_provider,
            is_flexible=is_flexible,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self._me.protect_content,
            message_effect_id=message_effect_id,
            reply_parameters=self.__reply_param,
            reply_markup=reply_markup,
        )

    async def reply_sticker(
        self: "tgram.types.Message",
        sticker: Union[Path, bytes, str],
        emoji: str = None,
        message_thread_id: int = None,
        disable_notification: bool = None,
        protect_content: bool = None,
        message_effect_id: str = None,
        reply_markup: Union[
            "tgram.types.InlineKeyboardMarkup",
            "tgram.types.ReplyKeyboardMarkup",
            "tgram.types.ReplyKeyboardRemove",
            "tgram.types.ForceReply",
        ] = None,
    ) -> "tgram.types.Message":
        """
        Sends a sticker in reply to the current message.

        Args:
            sticker (Union[Path, bytes, str]): The sticker to be sent. Can be a file path, bytes, or a URL.
            emoji (str, optional): Emoji associated with the sticker.
            message_thread_id (int, optional): Unique identifier for the target message thread (topic) of the forum; for forum supergroups only.
            disable_notification (bool, optional): Sends the message silently. Users will receive a notification with no sound.
            protect_content (bool, optional): Protects the contents of the sent message from forwarding and saving.
            message_effect_id (str, optional): Identifier of the message effect.
            reply_markup (Union[tgram.types.InlineKeyboardMarkup, tgram.types.ReplyKeyboardMarkup, tgram.types.ReplyKeyboardRemove, tgram.types.ForceReply], optional): Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove reply keyboard or to force a reply from the user.

        Returns:
            tgram.types.Message: The sent message.

        Example:
            ```python
            await message.reply_sticker(
                sticker="path/to/sticker.webp",
                emoji="😊",
                disable_notification=True
            )
            ```
        """
        return await self._me.send_sticker(
            self.chat.id,
            sticker=sticker,
            business_connection_id=self.business_connection_id,
            message_thread_id=message_thread_id,
            emoji=emoji,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self._me.protect_content,
            message_effect_id=message_effect_id,
            reply_parameters=self.__reply_param,
            reply_markup=reply_markup,
        )

    async def forward(
        self: "tgram.types.Message",
        chat_id: Union[int, str],
        message_thread_id: int = None,
        disable_notification: bool = None,
        protect_content: bool = None,
    ) -> "tgram.types.Message":
        """
        Forwards the current message to another chat.

        Args:
            chat_id (Union[int, str]): Unique identifier for the target chat or username of the target channel (in the format @channelusername).
            message_thread_id (int, optional): Unique identifier for the target message thread (topic) of the forum; for forum supergroups only.
            disable_notification (bool, optional): Sends the message silently. Users will receive a notification with no sound.
            protect_content (bool, optional): Protects the contents of the forwarded message from forwarding and saving.

        Returns:
            tgram.types.Message: The forwarded message.

        Example:
            ```python
            await message.forward(
                chat_id="@target_channel",
                disable_notification=True
            )
            ```
        """
        return await self._me.forward_message(
            chat_id=chat_id,
            from_chat_id=self.chat.id,
            message_id=self.id,
            message_thread_id=message_thread_id,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self._me.protect_content,
        )

    async def reply_media_from_file_id(
        self: "tgram.types.Message",
        file_id: str,
        caption: str = None,
        message_thread_id: int = None,
        parse_mode: "tgram.types.ParseMode" = None,
        caption_entities: List["tgram.types.MessageEntity"] = None,
        show_caption_above_media: bool = None,
        disable_notification: bool = None,
        protect_content: bool = None,
        message_effect_id: str = None,
        reply_markup: Union[
            "tgram.types.InlineKeyboardMarkup",
            "tgram.types.ReplyKeyboardMarkup",
            "tgram.types.ReplyKeyboardRemove",
            "tgram.types.ForceReply",
        ] = None,
    ) -> "tgram.types.Message":
        """
        Sends a media message using a file_id in reply to the current message.

        Args:
            file_id (str): The file_id of the media to be sent.
            caption (str, optional): Caption for the media.
            message_thread_id (int, optional): Unique identifier for the target message thread (topic) of the forum; for forum supergroups only.
            parse_mode (tgram.types.ParseMode, optional): Mode for parsing entities in the media caption.
            caption_entities (List[tgram.types.MessageEntity], optional): List of special entities that appear in the caption, which can be specified instead of parse_mode.
            show_caption_above_media (bool, optional): Whether to show the caption above the media.
            disable_notification (bool, optional): Sends the message silently. Users will receive a notification with no sound.
            protect_content (bool, optional): Protects the contents of the sent message from forwarding and saving.
            message_effect_id (str, optional): Identifier of the message effect.
            reply_markup (Union[tgram.types.InlineKeyboardMarkup, tgram.types.ReplyKeyboardMarkup, tgram.types.ReplyKeyboardRemove, tgram.types.ForceReply], optional): Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove reply keyboard or to force a reply from the user.

        Returns:
            tgram.types.Message: The sent message.

        Example:
            ```python
            await message.reply_media_from_file_id(
                file_id="ABC123",
                caption="Check out this media!",
                parse_mode="Markdown",
                disable_notification=True
            )
            ```
        """
        return await self._me.send_media_from_file_id(
            self.chat.id,
            file_id=file_id,
            business_connection_id=self.business_connection_id,
            message_thread_id=message_thread_id,
            caption=caption,
            parse_mode=parse_mode,
            caption_entities=caption_entities,
            show_caption_above_media=show_caption_above_media,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self._me.protect_content,
            message_effect_id=message_effect_id,
            reply_parameters=self.__reply_param,
            reply_markup=reply_markup,
        )

    async def copy(
        self: "tgram.types.Message",
        chat_id: Union[int, str],
        caption: str = None,
        message_thread_id: int = None,
        parse_mode: "tgram.types.ParseMode" = None,
        caption_entities: List["tgram.types.MessageEntity"] = None,
        show_caption_above_media: bool = None,
        disable_notification: bool = None,
        protect_content: bool = None,
        reply_parameters: "tgram.types.ReplyParameters" = None,
        reply_markup: Union[
            "tgram.types.InlineKeyboardMarkup",
            "tgram.types.ReplyKeyboardMarkup",
            "tgram.types.ReplyKeyboardRemove",
            "tgram.types.ForceReply",
        ] = None,
    ) -> "tgram.types.MessageId":
        """
        Copies the current message to another chat.

        Args:
            chat_id (Union[int, str]): Unique identifier for the target chat or username of the target channel (in the format @channelusername).
            caption (str, optional): New caption for the message, if any.
            message_thread_id (int, optional): Unique identifier for the target message thread (topic) of the forum; for forum supergroups only.
            parse_mode (tgram.types.ParseMode, optional): Mode for parsing entities in the message caption.
            caption_entities (List[tgram.types.MessageEntity], optional): List of special entities that appear in the caption, which can be specified instead of parse_mode.
            show_caption_above_media (bool, optional): Whether to show the caption above the media.
            disable_notification (bool, optional): Sends the message silently. Users will receive a notification with no sound.
            protect_content (bool, optional): Protects the contents of the copied message from forwarding and saving.
            reply_parameters (tgram.types.ReplyParameters, optional): Additional parameters for the reply.
            reply_markup (Union[tgram.types.InlineKeyboardMarkup, tgram.types.ReplyKeyboardMarkup, tgram.types.ReplyKeyboardRemove, tgram.types.ForceReply], optional): Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove reply keyboard or to force a reply from the user.

        Returns:
            tgram.types.MessageId: The ID of the copied message.

        Example:
            ```python
            await message.copy(
                chat_id="@target_channel",
                caption="Check out this copied message!",
                disable_notification=True
            )
            ```
        """
        return await self._me.copy_message(
            chat_id=chat_id,
            from_chat_id=self.chat.id,
            message_id=self.id,
            message_thread_id=message_thread_id,
            caption=caption,
            parse_mode=parse_mode,
            caption_entities=caption_entities,
            show_caption_above_media=show_caption_above_media,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self._me.protect_content,
            reply_parameters=reply_parameters,
            reply_markup=reply_markup,
        )

    async def edit_text(
        self: "tgram.types.Message",
        text: str,
        parse_mode: "tgram.types.ParseMode" = None,
        entities: List["tgram.types.MessageEntity"] = None,
        link_preview_options: "tgram.types.LinkPreviewOptions" = None,
        reply_markup: "tgram.types.InlineKeyboardMarkup" = None,
    ) -> Union["tgram.types.Message", "bool"]:
        """
        Edit the text of a message.

        Parameters:
        self (tgram.types.Message): The message object itself.
        text (str): New text of the message.
        parse_mode (tgram.types.ParseMode, optional): Mode for parsing entities in the message text. Defaults to None.
        entities (List[tgram.types.MessageEntity], optional): List of special entities that appear in message text. Defaults to None.
        link_preview_options (tgram.types.LinkPreviewOptions, optional): Options for link previews in the message. Defaults to None.
        reply_markup (tgram.types.InlineKeyboardMarkup, optional): Inline keyboard attached to the message. Defaults to None.

        Returns:
        Union[tgram.types.Message, bool]: On success, the edited message is returned, otherwise True is returned.

        Example:
        ```python
        message = await bot.send_message(chat_id, "Original text")
        edited_message = await message.edit_text(
            text="Edited text",
            parse_mode=tgram.types.ParseMode.MARKDOWN,
            entities=[tgram.types.MessageEntity(type="bold", offset=0, length=5)],
            link_preview_options=tgram.types.LinkPreviewOptions(disable_web_page_preview=True),
            reply_markup=tgram.types.InlineKeyboardMarkup(inline_keyboard=[[tgram.types.InlineKeyboardButton(text="Button", callback_data="data")]])
        ```
        """
        return await self._me.edit_message_text(
            text=text,
            business_connection_id=self.business_connection_id,
            chat_id=self.chat.id,
            message_id=self.id,
            parse_mode=parse_mode,
            entities=entities,
            link_preview_options=link_preview_options or self._me.link_preview_options,
            reply_markup=reply_markup,
        )

    edit = edit_text

    async def edit_caption(
        self: "tgram.types.Message",
        caption: str = None,
        parse_mode: "tgram.types.ParseMode" = None,
        caption_entities: List["tgram.types.MessageEntity"] = None,
        show_caption_above_media: bool = None,
        reply_markup: "tgram.types.InlineKeyboardMarkup" = None,
    ) -> Union["tgram.types.Message", "bool"]:
        """
        Edit the caption of a message.

        Args:
            caption (str, optional): New caption of the message. Defaults to None.
            parse_mode (tgram.types.ParseMode, optional): Mode for parsing entities in the message caption. Defaults to None.
            caption_entities (List[tgram.types.MessageEntity], optional): List of special entities that appear in the caption, which can be specified instead of parse_mode. Defaults to None.
            show_caption_above_media (bool, optional): Whether to show the caption above the media. Defaults to None.
            reply_markup (tgram.types.InlineKeyboardMarkup, optional): Inline keyboard attached to the message. Defaults to None.

        Returns:
            Union[tgram.types.Message, bool]: On success, the edited message is returned, otherwise True is returned.

        Example:
            ```python
            message = await bot.send_photo(chat_id, photo="path/to/photo.jpg", caption="Original caption")
            edited_message = await message.edit_caption(
                caption="Edited caption",
                parse_mode=tgram.types.ParseMode.MARKDOWN,
                caption_entities=[tgram.types.MessageEntity(type="bold", offset=0, length=6)],
                show_caption_above_media=True,
                reply_markup=tgram.types.InlineKeyboardMarkup(inline_keyboard=[[tgram.types.InlineKeyboardButton(text="Button", callback_data="data")]])
            )
            ```
        """
        return await self._me.edit_message_caption(
            business_connection_id=self.business_connection_id,
            chat_id=self.chat.id,
            message_id=self.id,
            caption=caption,
            parse_mode=parse_mode,
            caption_entities=caption_entities,
            show_caption_above_media=show_caption_above_media,
            reply_markup=reply_markup,
        )

    async def edit_reply_markup(
        self: "tgram.types.Message",
        reply_markup: "tgram.types.InlineKeyboardMarkup" = None,
    ) -> Union["tgram.types.Message", "bool"]:
        """
        Edit the reply markup of a message.

        Args:
            reply_markup (tgram.types.InlineKeyboardMarkup, optional): Inline keyboard attached to the message. Defaults to None.

        Returns:
            Union[tgram.types.Message, bool]: On success, the edited message is returned, otherwise True is returned.

        Example:
            ```python
            message = await bot.send_message(chat_id, "Original text")
            edited_message = await message.edit_reply_markup(
                reply_markup=tgram.types.InlineKeyboardMarkup(
                    inline_keyboard=[
                        [tgram.types.InlineKeyboardButton(text="Button", callback_data="data")]
                    ]
                )
            )
            ```
        """
        return await self._me.edit_message_reply_markup(
            business_connection_id=self.business_connection_id,
            chat_id=self.chat.id,
            message_id=self.id,
            reply_markup=reply_markup,
        )

    async def edit_media(
        self: "tgram.types.Message",
        media: "tgram.types.InputMedia",
        reply_markup: "tgram.types.InlineKeyboardMarkup" = None,
    ) -> Union["tgram.types.Message", "bool"]:
        """
        Edit the media content of a message.

        Args:
            media (tgram.types.InputMedia): The new media content to replace the current media.
            reply_markup (tgram.types.InlineKeyboardMarkup, optional): Inline keyboard attached to the message. Defaults to None.

        Returns:
            Union[tgram.types.Message, bool]: On success, the edited message is returned, otherwise True is returned.

        Example:
            ```python
            new_media = tgram.types.InputMediaPhoto(media="path/to/new_photo.jpg")
            edited_message = await message.edit_media(
                media=new_media,
                reply_markup=tgram.types.InlineKeyboardMarkup(
                    inline_keyboard=[
                        [tgram.types.InlineKeyboardButton(text="Button", callback_data="data")]
                    ]
                )
            )
            ```
        """
        return await self._me.edit_message_media(
            media=media,
            business_connection_id=self.business_connection_id,
            chat_id=self.chat.id,
            message_id=self.id,
            reply_markup=reply_markup,
        )

    async def edit_live_location(
        self: "tgram.types.Message",
        latitude: float,
        longitude: float,
        live_period: int = None,
        horizontal_accuracy: float = None,
        heading: int = None,
        proximity_alert_radius: int = None,
        reply_markup: "tgram.types.InlineKeyboardMarkup" = None,
    ) -> Union["tgram.types.Message", "bool"]:
        """
        Edits the live location of a message.

        Args:
            latitude (float): Latitude of the new location.
            longitude (float): Longitude of the new location.
            live_period (int, optional): Period in seconds for which the location will be updated (should be between 60 and 86400).
            horizontal_accuracy (float, optional): The radius of uncertainty for the location, measured in meters; 0-1500.
            heading (int, optional): For live locations, a direction in which the user is moving, in degrees; 1-360.
            proximity_alert_radius (int, optional): For live locations, a maximum distance for proximity alerts about approaching another chat member, in meters; 1-100000.
            reply_markup (tgram.types.InlineKeyboardMarkup, optional): Inline keyboard attached to the message.

        Returns:
            Union[tgram.types.Message, bool]: On success, the edited message is returned, otherwise True is returned.

        Example:
            ```python
            await message.edit_live_location(
                latitude=37.7749,
                longitude=-122.4194,
                live_period=3600,
                horizontal_accuracy=50,
                heading=90,
                proximity_alert_radius=100,
                reply_markup=tgram.types.InlineKeyboardMarkup(
                    inline_keyboard=[
                        [tgram.types.InlineKeyboardButton(text="Stop", callback_data="stop")]
                    ]
                )
            )
            ```
        """
        return await self._me.edit_message_live_location(
            latitude=latitude,
            longitude=longitude,
            business_connection_id=self.business_connection_id,
            chat_id=self.chat.id,
            message_id=self.id,
            live_period=live_period,
            horizontal_accuracy=horizontal_accuracy,
            heading=heading,
            proximity_alert_radius=proximity_alert_radius,
            reply_markup=reply_markup,
        )

    async def react(
        self: "tgram.types.Message",
        reaction: Union[
            List["tgram.types.ReactionType"], "tgram.types.ReactionType", List[str], str
        ],
    ) -> bool:
        """
        Adds a reaction to the current message.

        Args:
            reaction (Union[List[tgram.types.ReactionType], tgram.types.ReactionType, List[str], str]):
                The reaction(s) to be added. Can be a list of ReactionType objects, a single ReactionType object,
                a list of emoji strings, or a single emoji string.

        Returns:
            bool: True on success.

        Example:
            ```python
            # Adding a single reaction using an emoji string
            await message.react(reaction="👍")

            # Adding multiple reactions using emoji strings
            await message.react(reaction=["👍", "❤️"])

            # Adding a single reaction using a ReactionType object
            reaction_type = tgram.types.ReactionTypeEmoji("👍")
            await message.react(reaction=reaction_type)

            # Adding multiple reactions using ReactionType objects
            reaction_types = [tgram.types.ReactionTypeEmoji("👍"), tgram.types.ReactionTypeEmoji("❤️")]
            await message.react(reaction=reaction_types)
            ```
        """
        reactions = (
            [
                tgram.types.ReactionTypeEmoji(i) if isinstance(i, str) else i
                for i in reaction
            ]
            if isinstance(reaction, list)
            else [
                tgram.types.ReactionTypeEmoji(reaction)
                if isinstance(reaction, str)
                else reaction
            ]
        )
        return await self._me.set_message_reaction(
            self.chat.id,
            self.id,
            reaction=reactions,
        )

    async def download(
        self: "tgram.types.Message", file_path: str = None, in_memory: bool = None
    ) -> Union[Path, BytesIO]:
        """
        Downloads the media content of the message.

        Args:
            file_path (str, optional): The path where the file should be saved. If not provided, the file will be saved in the current directory with its original name.
            in_memory (bool, optional): If True, the file will be downloaded into memory and returned as a BytesIO object. If False or not provided, the file will be saved to disk.

        Returns:
            Union[Path, BytesIO]: The path to the saved file or a BytesIO object containing the file data.

        Raises:
            ValueError: If the message does not contain any media.

        Example:
            ```python
            # Downloading a photo to a specific path
            await message.download(file_path="downloads/photo.jpg")

            # Downloading a video into memory
            video_data = await message.download(in_memory=True)
            with open("downloads/video.mp4", "wb") as f:
                f.write(video_data.getbuffer())
            ```
        """
        if not self.media:
            raise ValueError("This message has no media to download.")

        media = getattr(self, self.media)
        file_id = media[-1].file_id if isinstance(media, list) else media.file_id
        return await self._me.download_file(
            file_id, file_path=file_path, in_memory=in_memory
        )

    async def delete(self: "tgram.types.Message") -> bool:
        """
        Deletes the current message.

        Args:
            self (tgram.types.Message): The message object itself.

        Returns:
            bool: True on success.

        Example:
            ```python
            # Assuming 'message' is an instance of tgram.types.Message
            success = await message.delete()
            if success:
                print("Message deleted successfully.")
            else:
                print("Failed to delete the message.")
            ```
        """
        return await self._me.delete_message(self.chat.id, self.id)

    @property
    def id(self: "tgram.types.Message") -> int:
        return self.message_id

    @property
    def media(
        self: "tgram.types.Message",
    ) -> Optional[
        Literal[
            "audio",
            "video",
            "photo",
            "animation",
            "voice",
            "video_note",
            "sticker",
            "document",
        ]
    ]:
        """
        Returns the type of media contained in the message, if any.

        This property checks the message for various types of media (e.g., audio, video, photo, etc.) and returns the type of media if found.

        Returns:
            Optional[Literal["audio", "video", "photo", "animation", "voice", "video_note", "sticker", "document"]]: The type of media contained in the message, or None if no media is found.

        Example:
            ```python
            media_type = message.media
            if media_type:
                print(f"The message contains {media_type}.")
            else:
                print("The message does not contain any media.")
            ```
        """
        for media_type in MEDIA_TYPES:
            if getattr(self, media_type):
                return media_type

        return None

    @property
    def service(
        self: "tgram.types.Message",
    ) -> Optional[str]:
        """
        Returns the type of service message contained in the message, if any.

        This property checks the message for various types of service messages (e.g., video chat started, pinned message, etc.) and returns the type of service message if found.

        Returns:
            Optional[str]: The type of service message contained in the message, or None if no service message is found.

        Example:
            ```python
            service_type = message.service
            if service_type:
                print(f"The message contains a service message of type {service_type}.")
            else:
                print("The message does not contain any service message.")
            ```
        """
        for service_type in SERVICE_TYPES:
            if getattr(self, service_type):
                return service_type

        return None

    @property
    def link(self: "tgram.types.Message") -> Optional[str]:
        """
        Generates a link to the message.

        This property generates a link to the message based on the chat type and chat username or ID.

        Returns:
            Optional[str]: The link to the message, or None if the chat type is private.

        Example:
            ```python
            message_link = message.link
            if message_link:
                print(f"Message link: {message_link}")
            else:
                print("This message is in a private chat and does not have a link.")
            ```
        """
        if self.chat.type == "private":
            return None

        chat_id = self.chat.username or str(self.chat.id).replace("-100", "")
        return f"https://t.me/{chat_id}/{self.id}"

    @property
    def file_id(self: "tgram.types.Message") -> str:
        """
        Returns the file_id of the media contained in the message.

        This property checks the message for various types of media (e.g., audio, video, photo, etc.) and returns the file_id of the media if found.

        Raises:
            ValueError: If the message does not contain any media.

        Returns:
            str: The file_id of the media contained in the message.

        Example:
            ```python
            file_id = message.file_id
            print(f"The file_id of the media is: {file_id}")
            ```
        """
        media_type = self.media

        if media_type is None:
            raise ValueError("This message has no media.")

        media = getattr(self, media_type)

        if isinstance(media, list):
            return media[-1].file_id
        else:
            return media.file_id

    @property
    def user(self: "tgram.types.Message") -> Optional["tgram.types.User"]:
        """
        Returns the user who sent the message.

        This property retrieves the user who sent the message, if available.

        Returns:
            Optional[tgram.types.User]: The user who sent the message, or None if the user is not available.

        Example:
            ```python
            user = message.user
            if user:
                print(f"Message sent by: {user.first_name}")
            else:
                print("User information is not available.")
            ```
        """
        return self.from_user

    sender_user = user

    @property
    def __reply_param(self) -> "tgram.types.ReplyParameters":
        return tgram.types.ReplyParameters(self.id, allow_sending_without_reply=True)



import tgram
from tgram.types import InlineKeyboardMarkup, Message, InputChecklist

from tgram.utils import convert_to_inline_keyboard_markup


class EditMessageChecklist:
    async def edit_message_checklist(
        self: "tgram.TgBot",
        business_connection_id: str,
        chat_id: int,
        message_id: int,
        checklist: InputChecklist,
        reply_markup: InlineKeyboardMarkup = None,
    ) -> Message:
        """
        Use this method to edit a checklist on behalf of a connected business account.
        On success, the edited Message is returned.

        Telegram documentation: https://core.telegram.org/bots/api#editmessagechecklist

        :param business_connection_id: Unique identifier of the business connection on behalf of which the message will be sent
        :type business_connection_id: :obj:`str`

        :param chat_id: Unique identifier for the target chat
        :type chat_id: :obj:`int`

        :param message_id: Unique identifier for the target message
        :type message_id: :obj:`int`

        :param checklist: A JSON-serialized object for the new checklist
        :type checklist: :class:`tgram.types.InputChecklist`

        :param reply_markup: A JSON-serialized object for the new inline keyboard for the message
        :type reply_markup: :class:`tgram.types.InlineKeyboardMarkup`

        :return: On success, the edited Message is returned.
        :rtype: :class:`tgram.types.Message`
        """
        result = await self(
            "editMessageChecklist",
            business_connection_id=business_connection_id,
            chat_id=chat_id,
            message_id=message_id,
            checklist=checklist,
            reply_markup=convert_to_inline_keyboard_markup(reply_markup)
            if isinstance(reply_markup, list)
            else reply_markup,
        )
        return Message._parse(me=self, d=result["result"])



import tgram
from .type_ import Type_

from typing import Optional


class BusinessBotRights(Type_):
    """

    Represents the rights of a business bot.

    Telegram Documentation: https://core.telegram.org/bots/api#businessbotrights

    :param can_reply: Optional. True, if the bot can send and edit messages in the private chats that had incoming messages in the last 24 hours.
    :type can_reply: bool, optional
    :param can_read_messages: Optional. True, if the bot can mark incoming private messages as read.
    :type can_read_messages: bool, optional
    :param can_delete_sent_messages: Optional. True, if the bot can delete messages sent by the bot.
    :type can_delete_sent_messages: bool, optional
    :param can_delete_all_messages: Optional. True, if the bot can delete all private messages in managed chats.
    :type can_delete_all_messages: bool, optional
    :param can_edit_name: Optional. True, if the bot can edit the first and last name of the business account.
    :type can_edit_name: bool, optional
    :param can_edit_bio: Optional. True, if the bot can edit the bio of the business account.
    :type can_edit_bio: bool, optional
    :param can_edit_profile_photo: Optional. True, if the bot can edit the profile photo of the business account.
    :type can_edit_profile_photo: bool, optional
    :param can_edit_username: Optional. True, if the bot can edit the username of the business account.
    :type can_edit_username: bool, optional
    :param can_change_gift_settings: Optional. True, if the bot can change the privacy settings pertaining to gifts for the business account.
    :type can_change_gift_settings: bool, optional
    :param can_view_gifts_and_stars: Optional. True, if the bot can view gifts and the amount of Telegram Stars owned by the business account.
    :type can_view_gifts_and_stars: bool, optional
    :param can_convert_gifts_to_stars: Optional. True, if the bot can convert regular gifts owned by the business account to Telegram Stars.
    :type can_convert_gifts_to_stars: bool, optional
    :param can_transfer_and_upgrade_gifts: Optional. True, if the bot can transfer and upgrade gifts owned by the business account.
    :type can_transfer_and_upgrade_gifts: bool, optional
    :param can_transfer_stars: Optional. True, if the bot can transfer Telegram Stars received by the business account to its own account, or use them to upgrade and transfer gifts.
    :type can_transfer_stars: bool, optional
    :param can_manage_stories: Optional. True, if the bot can post, edit and delete stories on behalf of the business account.
    :type can_manage_stories: bool, optional

    :return: Instance of the class
    :rtype: :class:`tgram.types.BusinessBotRights`
    """

    def __init__(
        self,
        can_reply: "bool" = None,
        can_read_messages: "bool" = None,
        can_delete_sent_messages: "bool" = None,
        can_delete_all_messages: "bool" = None,
        can_edit_name: "bool" = None,
        can_edit_bio: "bool" = None,
        can_edit_profile_photo: "bool" = None,
        can_edit_username: "bool" = None,
        can_change_gift_settings: "bool" = None,
        can_view_gifts_and_stars: "bool" = None,
        can_convert_gifts_to_stars: "bool" = None,
        can_transfer_and_upgrade_gifts: "bool" = None,
        can_transfer_stars: "bool" = None,
        can_manage_stories: "bool" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.can_reply = can_reply
        self.can_read_messages = can_read_messages
        self.can_delete_sent_messages = can_delete_sent_messages
        self.can_delete_all_messages = can_delete_all_messages
        self.can_edit_name = can_edit_name
        self.can_edit_bio = can_edit_bio
        self.can_edit_profile_photo = can_edit_profile_photo
        self.can_edit_username = can_edit_username
        self.can_change_gift_settings = can_change_gift_settings
        self.can_view_gifts_and_stars = can_view_gifts_and_stars
        self.can_convert_gifts_to_stars = can_convert_gifts_to_stars
        self.can_transfer_and_upgrade_gifts = can_transfer_and_upgrade_gifts
        self.can_transfer_stars = can_transfer_stars
        self.can_manage_stories = can_manage_stories

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.BusinessBotRights"]:
        return (
            BusinessBotRights(
                me=me,
                json=d,
                can_reply=d.get("can_reply"),
                can_read_messages=d.get("can_read_messages"),
                can_delete_sent_messages=d.get("can_delete_sent_messages"),
                can_delete_all_messages=d.get("can_delete_all_messages"),
                can_edit_name=d.get("can_edit_name"),
                can_edit_bio=d.get("can_edit_bio"),
                can_edit_profile_photo=d.get("can_edit_profile_photo"),
                can_edit_username=d.get("can_edit_username"),
                can_change_gift_settings=d.get("can_change_gift_settings"),
                can_view_gifts_and_stars=d.get("can_view_gifts_and_stars"),
                can_convert_gifts_to_stars=d.get("can_convert_gifts_to_stars"),
                can_transfer_and_upgrade_gifts=d.get("can_transfer_and_upgrade_gifts"),
                can_transfer_stars=d.get("can_transfer_stars"),
                can_manage_stories=d.get("can_manage_stories"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram
from .base import StorageBase

from typing import Any, Dict, List, Tuple, Union


class KvsqliteStorage(StorageBase):
    def __init__(self, bot: "tgram.TgBot", client=None) -> None:
        super().__init__(bot, "kvsqlite", client)

    async def set(self, key: str, value: Any) -> bool:
        return await self.client.set(key, value)

    async def get(self, key: str) -> Any:
        return await self.client.get(key)

    async def add_chat(self, chat: "tgram.types.Chat") -> bool:
        chat_json = chat.json
        chats = await self.get_chats()
        if chat.username:
            chats.update({chat.username.lower(): chat.id})
        chats.update({chat.id: chat_json})
        return await self.update_chats(chats)

    async def get_chat(
        self, chat_id: Union[int, str], parse: bool = False
    ) -> Union[dict, "tgram.types.Chat"]:
        chats = await self.get_chats()

        if chat := chats.get(chat_id.lower() if isinstance(chat_id, str) else chat_id):
            if isinstance(chat, int):
                return await self.get_chat(chat, parse)
            return tgram.types.Chat._parse(self.bot, chat) if parse else chat

        return {}

    async def get_chats(self) -> Dict[str, dict]:
        return await self.client.get("chats") or {}

    async def update_chats(self, chats: Dict[str, dict]) -> bool:
        return await self.client.set("chats", chats)

    async def add_user(self, user: "tgram.types.User") -> bool:
        user_json = user.json
        users = await self.get_users()
        if user.username:
            users.update({user.username.lower(): user.id})
        users.update({user.id: user_json})
        return await self.update_users(users)

    async def get_user(
        self, user_id: Union[int, str], parse: bool = False
    ) -> Union[dict, "tgram.types.User"]:
        users = await self.get_users()

        if user := users.get(user_id.lower() if isinstance(user_id, str) else user_id):
            if isinstance(user, int):
                return await self.get_user(user, parse)
            return tgram.types.User._parse(self.bot, user) if parse else user

        return {}

    async def get_users(self) -> Dict[str, Dict]:
        return await self.client.get("users") or {}

    async def update_users(self, users: Dict[str, dict]) -> bool:
        return await self.client.set("users", users)

    async def mute(self, chat_id: int, user_id: int) -> bool:
        mute_list = await self.get_mute_list()
        packet = (chat_id, user_id)
        if packet in mute_list:
            return False
        mute_list.append(packet)
        return await self.update_mute_list(mute_list)

    async def unmute(self, chat_id: int, user_id: int) -> bool:
        mute_list = await self.get_mute_list()
        packet = (chat_id, user_id)
        if packet not in mute_list:
            return False
        mute_list.remove(packet)
        return await self.update_mute_list(mute_list)

    async def get_mute_list(self) -> List[Tuple[int, int]]:
        return await self.get("mute") or []

    async def update_mute_list(self, mute_list: List[Tuple[int, int]]) -> bool:
        return await self.set("mute", mute_list)



import tgram
from .type_ import Type_

from typing import List, Optional
from tgram.utils import String


class TextQuote(Type_):
    """
    This object contains information about the quoted part of a message that is replied to by the given message.

    Telegram documentation: https://core.telegram.org/bots/api#textquote

    :param text: Text of the quoted part of a message that is replied to by the given message
    :type text: :class:`tgram.utils.String`

    :param entities: Optional. Special entities that appear in the quote. Currently, only bold, italic, underline, strikethrough, spoiler, and custom_emoji entities are kept in quotes.
    :type entities: :obj:`list` of :class:`MessageEntity`

    :param position: Approximate quote position in the original message in UTF-16 code units as specified by the sender
    :type position: :obj:`int`

    :param is_manual: Optional. True, if the quote was chosen manually by the message sender. Otherwise, the quote was added automatically by the server.
    :type is_manual: :obj:`bool`

    :return: Instance of the class
    :rtype: :class:`TextQuote`
    """

    def __init__(
        self,
        text: "String" = None,
        position: "int" = None,
        entities: List["tgram.types.MessageEntity"] = None,
        is_manual: "bool" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.text = String(text).put(entities)
        self.entities = entities
        self.position = position
        self.is_manual = is_manual

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.TextQuote"]:
        return (
            TextQuote(
                me=me,
                json=d,
                text=d.get("text"),
                position=d.get("position"),
                entities=[
                    tgram.types.MessageEntity._parse(me=me, d=i)
                    for i in d.get("entities")
                ]
                if d.get("entities")
                else None,
                is_manual=d.get("is_manual"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram
from .type_ import Type_

from typing import Optional


class GameHighScore(Type_):
    """
    This object represents one row of the high scores table for a game.

    Telegram Documentation: https://core.telegram.org/bots/api#gamehighscore

    :param position: Position in high score table for the game
    :type position: :obj:`int`

    :param user: User
    :type user: :class:`tgram.types.User`

    :param score: Score
    :type score: :obj:`int`

    :return: Instance of the class
    :rtype: :class:`tgram.types.GameHighScore`
    """

    def __init__(
        self,
        position: "int" = None,
        user: "tgram.types.User" = None,
        score: "int" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.position = position
        self.user = user
        self.score = score

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.GameHighScore"]:
        return (
            GameHighScore(
                me=me,
                json=d,
                position=d.get("position"),
                user=tgram.types.User._parse(me=me, d=d.get("user")),
                score=d.get("score"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram


class DeleteStickerSet:
    async def delete_sticker_set(self: "tgram.TgBot", name: str) -> bool:
        """
        Use this method to delete a sticker set. Returns True on success.

        :param name: Sticker set name
        :type name: :obj:`str`

        :return: Returns True on success.
        :rtype: :obj:`bool`
        """

        result = await self(
            "deleteStickerSet",
            name=name,
        )
        return result["result"]



import tgram
from .type_ import Type_

from typing import Optional


class ReactionCount(Type_):
    """
    This object represents a reaction added to a message along with the number of times it was added.

    Telegram documentation: https://core.telegram.org/bots/api#reactioncount

    :param type: Type of the reaction
    :type type: :class:`ReactionType`

    :param total_count: Number of times the reaction was added
    :type total_count: :obj:`int`

    :return: Instance of the class
    :rtype: :class:`ReactionCount`
    """

    def __init__(
        self,
        type: "tgram.types.ReactionType" = None,
        total_count: "int" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.type = type
        self.total_count = total_count

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.ReactionCount"]:
        return (
            ReactionCount(
                me=me,
                json=d,
                type=(
                    tgram.types.ReactionTypeCustomEmoji._parse(me, d.get("type"))
                    if d["type"]["type"] == "custom_emoji"
                    else tgram.types.ReactionTypeEmoji._parse(me, d.get("type"))
                    if d["type"]["type"] == "emoji"
                    else tgram.types.ReactionTypePaid._parse(me, d.get("type"))
                )
                if d.get("type")
                else None,
                total_count=d.get("total_count"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram


class SetBusinessAccountGiftSettings:
    async def set_business_account_gift_settings(
        self: "tgram.TgBot",
        business_connection_id: str,
        show_gift_button: bool,
        accepted_gift_types: "tgram.types.AcceptedGiftTypes",
    ) -> bool:
        """
        Changes the privacy settings pertaining to incoming gifts in a managed business account.
        Requires the can_change_gift_settings business bot right.

        Telegram documentation: https://core.telegram.org/bots/api#setbusinessaccountgiftsettings

        :param business_connection_id: Unique identifier of the business connection
        :type business_connection_id: :obj:`str`

        :param show_gift_button: Pass True, if a button for sending a gift to the user or by the business account must always be shown in the input field
        :type show_gift_button: :obj:`bool`

        :param accepted_gift_types: Types of gifts accepted by the business account
        :type accepted_gift_types: :obj:`tgram.types.AcceptedGiftTypes`

        :return: On success, returns True.
        :rtype: :obj:`bool`
        """
        result = await self(
            "setBusinessAccountGiftSettings",
            business_connection_id=business_connection_id,
            show_gift_button=show_gift_button,
            accepted_gift_types=accepted_gift_types,
        )
        return result["result"]



import tgram
from .type_ import Type_

from typing import Optional


class LinkPreviewOptions(Type_):
    """
    Describes the options used for link preview generation.

    Telegram documentation: https://core.telegram.org/bots/api#linkpreviewoptions

    :param is_disabled: Optional. True, if the link preview is disabled
    :type is_disabled: :obj:`bool`

    :param url: Optional. URL to use for the link preview. If empty, then the first URL found in the message text will be used
    :type url: :obj:`str`

    :param prefer_small_media: Optional. True, if the media in the link preview is supposed to be shrunk; ignored if the URL isn't explicitly specified or media size change isn't supported for the preview
    :type prefer_small_media: :obj:`bool`

    :param prefer_large_media: Optional. True, if the media in the link preview is supposed to be enlarged; ignored if the URL isn't explicitly specified or media size change isn't supported for the preview
    :type prefer_large_media: :obj:`bool`

    :param show_above_text: Optional. True, if the link preview must be shown above the message text; otherwise, the link preview will be shown below the message text
    :type show_above_text: :obj:`bool`

    :return: Instance of the class
    :rtype: :class:`LinkPreviewOptions`
    """

    def __init__(
        self,
        is_disabled: "bool" = None,
        url: "str" = None,
        prefer_small_media: "bool" = None,
        prefer_large_media: "bool" = None,
        show_above_text: "bool" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.is_disabled = is_disabled
        self.url = url
        self.prefer_small_media = prefer_small_media
        self.prefer_large_media = prefer_large_media
        self.show_above_text = show_above_text

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.LinkPreviewOptions"]:
        return (
            LinkPreviewOptions(
                me=me,
                json=d,
                is_disabled=d.get("is_disabled"),
                url=d.get("url"),
                prefer_small_media=d.get("prefer_small_media"),
                prefer_large_media=d.get("prefer_large_media"),
                show_above_text=d.get("show_above_text"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram
from typing import List
from typing import Union
from tgram.types import ForceReply
from tgram.types import InlineKeyboardMarkup
from tgram.types import LinkPreviewOptions
from tgram.types import Message
from tgram.types import MessageEntity
from tgram.types import ReplyKeyboardMarkup
from tgram.types import ReplyKeyboardRemove
from tgram.types import ReplyParameters, ParseMode

from tgram.utils import get_parse_mode, convert_to_inline_keyboard_markup


class SendMessage:
    async def send_message(
        self: "tgram.TgBot",
        chat_id: Union[int, str],
        text: str,
        business_connection_id: str = None,
        message_thread_id: int = None,
        parse_mode: ParseMode = None,
        entities: List[MessageEntity] = None,
        link_preview_options: LinkPreviewOptions = None,
        disable_notification: bool = None,
        protect_content: bool = None,
        message_effect_id: str = None,
        reply_parameters: ReplyParameters = None,
        reply_markup: Union[
            InlineKeyboardMarkup, ReplyKeyboardMarkup, ReplyKeyboardRemove, ForceReply
        ] = None,
        allow_paid_broadcast: bool = None,
    ) -> Message:
        """
        Use this method to send text messages.

        Warning: Do not send more than about 4096 characters each message, otherwise you'll risk an HTTP 414 error.
        If you must send more than 4096 characters,
        use the `split_string` or `smart_split` function in util.py.

        Telegram documentation: https://core.telegram.org/bots/api#sendmessage

        :param chat_id: Unique identifier for the target chat or username of the target channel (in the format @channelusername)
        :type chat_id: :obj:`int` or :obj:`str`

        :param text: Text of the message to be sent
        :type text: :obj:`str`

        :param parse_mode: Mode for parsing entities in the message text.
        :type parse_mode: :obj:`str`

        :param entities: List of special entities that appear in message text, which can be specified instead of parse_mode
        :type entities: Array of :class:`tgram.types.MessageEntity`

        :param disable_web_page_preview: Deprecated - Use link_preview_options instead.
        :type disable_web_page_preview: :obj:`bool`

        :param disable_notification: Sends the message silently. Users will receive a notification with no sound.
        :type disable_notification: :obj:`bool`

        :param protect_content: If True, the message content will be hidden for all users except for the target user
        :type protect_content: :obj:`bool`

        :param reply_to_message_id: Deprecated - Use reply_parameters instead. If the message is a reply, ID of the original message
        :type reply_to_message_id: :obj:`int`

        :param allow_sending_without_reply: Deprecated - Use reply_parameters instead. Pass True, if the message should be sent even if the specified replied-to message is not found
        :type allow_sending_without_reply: :obj:`bool`

        :param reply_markup: Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove reply keyboard or to force a reply from the user.
        :type reply_markup: :class:`tgram.types.InlineKeyboardMarkup` or :class:`tgram.types.ReplyKeyboardMarkup` or :class:`tgram.types.ReplyKeyboardRemove`
            or :class:`tgram.types.ForceReply`

        :param timeout: Timeout in seconds for the request.
        :type timeout: :obj:`int`

        :param message_thread_id: Unique identifier for the target message thread (topic) of the forum; for forum supergroups only
        :type message_thread_id: :obj:`int`

        :param reply_parameters: Reply parameters.
        :type reply_parameters: :class:`tgram.types.ReplyParameters`

        :param link_preview_options: Options for previewing links.
        :type link_preview_options: :class:`tgram.types.LinkPreviewOptions`

        :param business_connection_id: Unique identifier for the target business connection
        :type business_connection_id: :obj:`str`

        :param message_effect_id: Unique identifier for the message effect
        :type message_effect_id: :obj:`str`

        :param allow_paid_broadcast: Pass True to allow up to 1000 messages per second, ignoring broadcasting limits for a fee of 0.1 Telegram Stars per message.
            The relevant Stars will be withdrawn from the bot's balance
        :type allow_paid_broadcast: :obj:`bool`

        :return: On success, the sent Message is returned.
        :rtype: :class:`tgram.types.Message`
        """

        result = await self(
            "sendMessage",
            chat_id=chat_id,
            text=text,
            business_connection_id=business_connection_id,
            message_thread_id=message_thread_id,
            parse_mode=get_parse_mode(self, parse_mode),
            entities=entities,
            link_preview_options=link_preview_options or self.link_preview_options,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self.protect_content,
            message_effect_id=message_effect_id,
            reply_parameters=reply_parameters,
            reply_markup=convert_to_inline_keyboard_markup(reply_markup)
            if isinstance(reply_markup, list)
            else reply_markup,
            allow_paid_broadcast=allow_paid_broadcast,
        )
        return Message._parse(me=self, d=result["result"])



import tgram
from .type_ import Type_
from typing import Optional


class InputStoryContentPhoto(Type_):
    """
    Describes a photo to post as a story.

    :param type: Type of the content, always "photo"
    :type type: :obj:`str`
    :param photo: The photo to post as a story. The photo must be of the size 1080x1920 and must not exceed 10 MB.
    :type photo: :obj:`str`
    """

    def __init__(
        self,
        photo: str = None,
        me: "tgram.TgBot" = None,
        json: dict = None,
    ):
        super().__init__(me=me, json=json)
        self.type = "photo"
        self.photo = photo

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["InputStoryContentPhoto"]:
        return (
            InputStoryContentPhoto(
                photo=d.get("photo"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )


class InputStoryContentVideo(Type_):
    """
    Describes a video to post as a story.

    :param type: Type of the content, always "video"
    :type type: :obj:`str`
    :param video: The video to post as a story. The video must be of the size 720x1280, streamable, encoded with H.265 codec, with key frames added each second in the MPEG4 format, and must not exceed 30 MB.
    :type video: :obj:`str`
    :param duration: Optional. Precise duration of the video in seconds; 0-60
    :type duration: :obj:`float`
    :param cover_frame_timestamp: Optional. Timestamp in seconds of the frame that will be used as the static cover for the story. Defaults to 0.0.
    :type cover_frame_timestamp: :obj:`float`
    :param is_animation: Optional. Pass True if the video has no sound
    :type is_animation: :obj:`bool`
    """

    def __init__(
        self,
        video: str = None,
        duration: float = None,
        cover_frame_timestamp: float = None,
        is_animation: bool = None,
        me: "tgram.TgBot" = None,
        json: dict = None,
    ):
        super().__init__(me=me, json=json)
        self.type = "video"
        self.video = video
        self.duration = duration
        self.cover_frame_timestamp = cover_frame_timestamp
        self.is_animation = is_animation

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["InputStoryContentVideo"]:
        return (
            InputStoryContentVideo(
                video=d.get("video"),
                duration=d.get("duration"),
                cover_frame_timestamp=d.get("cover_frame_timestamp"),
                is_animation=d.get("is_animation"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram
from typing import List
from typing import Union
from tgram.types import MessageId


class ForwardMessages:
    async def forward_messages(
        self: "tgram.TgBot",
        chat_id: Union[int, str],
        from_chat_id: Union[int, str],
        message_ids: List[int],
        message_thread_id: int = None,
        disable_notification: bool = None,
        protect_content: bool = None,
    ) -> List[MessageId]:
        """
        Use this method to forward multiple messages of any kind. If some of the specified messages can't be found or forwarded,
        they are skipped. Service messages and messages with protected content can't be forwarded.
        Album grouping is kept for forwarded messages. On success, an array of MessageId of the sent messages is returned.

        Telegram documentation: https://core.telegram.org/bots/api#forwardmessages

        :param chat_id: Unique identifier for the target chat or username of the target channel (in the format @channelusername)
        :type chat_id: :obj:`int` or :obj:`str`

        :param from_chat_id: Unique identifier for the chat where the original message was sent (or channel username in the format @channelusername)
        :type from_chat_id: :obj:`int` or :obj:`str`

        :param message_ids: Message identifiers in the chat specified in from_chat_id
        :type message_ids: :obj:`list`

        :param disable_notification: Sends the message silently. Users will receive a notification with no sound
        :type disable_notification: :obj:`bool`

        :param message_thread_id: Identifier of a message thread, in which the messages will be sent
        :type message_thread_id: :obj:`int`

        :param protect_content: Protects the contents of the forwarded message from forwarding and saving
        :type protect_content: :obj:`bool`

        :return: On success, the sent Message is returned.
        :rtype: :class:`tgram.types.MessageID`
        """

        result = await self(
            "forwardMessages",
            chat_id=chat_id,
            from_chat_id=from_chat_id,
            message_ids=message_ids,
            message_thread_id=message_thread_id,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self.protect_content,
        )
        return [MessageId._parse(me=self, d=i) for i in result["result"]]



import tgram
import random
from .type_ import Type_

from typing import Optional


class InlineQueryResultGame(Type_):
    """
    Represents a Game.

    Telegram Documentation: https://core.telegram.org/bots/api#inlinequeryresultgame

    :param type: Type of the result, must be game
    :type type: :obj:`str`

    :param id: Unique identifier for this result, 1-64 bytes
    :type id: :obj:`str`

    :param game_short_name: Short name of the game
    :type game_short_name: :obj:`str`

    :param reply_markup: Optional. Inline keyboard attached to the message
    :type reply_markup: :class:`tgram.types.InlineKeyboardMarkup`

    :return: Instance of the class
    :rtype: :class:`tgram.types.InlineQueryResultGame`
    """

    def __init__(
        self,
        game_short_name: "str" = None,
        reply_markup: "tgram.types.InlineKeyboardMarkup" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.type = "game"
        self.id = random.randint(10000, 99999)
        self.game_short_name = game_short_name
        self.reply_markup = reply_markup

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.InlineQueryResultGame"]:
        return (
            InlineQueryResultGame(
                me=me,
                json=d,
                game_short_name=d.get("game_short_name"),
                reply_markup=tgram.types.InlineKeyboardMarkup._parse(
                    me=me, d=d.get("reply_markup")
                ),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram
from typing import List
from tgram.types import ShippingOption


class AnswerShippingQuery:
    async def answer_shipping_query(
        self: "tgram.TgBot",
        shipping_query_id: str,
        ok: bool,
        shipping_options: List[ShippingOption] = None,
        error_message: str = None,
    ) -> bool:
        """
        Asks for an answer to a shipping question.

        Telegram documentation: https://core.telegram.org/bots/api#answershippingquery

        :param shipping_query_id: Unique identifier for the query to be answered
        :type shipping_query_id: :obj:`str`

        :param ok: Specify True if delivery to the specified address is possible and False if there are any problems (for example, if delivery to the specified address is not possible)
        :type ok: :obj:`bool`

        :param shipping_options: Required if ok is True. A JSON-serialized array of available shipping options.
        :type shipping_options: :obj:`list` of :obj:`ShippingOption`

        :param error_message: Required if ok is False. Error message in human readable form that explains why it is impossible to complete the order
            (e.g. "Sorry, delivery to your desired address is unavailable'). Telegram will display this message to the user.
        :type error_message: :obj:`str`

        :return: On success, True is returned.
        :rtype: :obj:`bool`
        """

        result = await self(
            "answerShippingQuery",
            shipping_query_id=shipping_query_id,
            ok=ok,
            shipping_options=shipping_options,
            error_message=error_message,
        )
        return result["result"]



import tgram
from tgram.types import OwnedGifts


class GetBusinessAccountGifts:
    async def get_business_account_gifts(
        self: "tgram.TgBot",
        business_connection_id: str,
        exclude_unsaved: bool = None,
        exclude_saved: bool = None,
        exclude_unlimited: bool = None,
        exclude_limited: bool = None,
        exclude_unique: bool = None,
        sort_by_price: bool = None,
        offset: str = "",
        limit: int = 100,
    ) -> OwnedGifts:
        """
        Use this method to get the gifts received and owned by a managed business account.
        Requires the can_view_gifts_and_stars business bot right.
        Returns an OwnedGifts object on success.

        Telegram documentation: https://core.telegram.org/bots/api#getbusinessaccountgifts

        :param business_connection_id: Unique identifier of the business connection
        :type business_connection_id: :obj:`str`
        :param exclude_unsaved: Exclude gifts that aren't saved to the account's profile page
        :type exclude_unsaved: :obj:`bool`, optional
        :param exclude_saved: Exclude gifts that are saved to the account's profile page
        :type exclude_saved: :obj:`bool`, optional
        :param exclude_unlimited: Exclude gifts that can be purchased an unlimited number of times
        :type exclude_unlimited: :obj:`bool`, optional
        :param exclude_limited: Exclude gifts that can be purchased a limited number of times
        :type exclude_limited: :obj:`bool`, optional
        :param exclude_unique: Exclude unique gifts
        :type exclude_unique: :obj:`bool`, optional
        :param sort_by_price: Sort results by gift price instead of send date
        :type sort_by_price: :obj:`bool`, optional
        :param offset: Offset of the first entry to return; use empty string for first chunk
        :type offset: :obj:`str`, optional
        :param limit: Maximum number of gifts to return; 1-100, defaults to 100
        :type limit: :obj:`int`, optional

        :return: Returns an OwnedGifts object on success.
        :rtype: :class:`tgram.types.OwnedGifts`
        """
        result = await self(
            "getBusinessAccountGifts",
            business_connection_id=business_connection_id,
            exclude_unsaved=exclude_unsaved,
            exclude_saved=exclude_saved,
            exclude_unlimited=exclude_unlimited,
            exclude_limited=exclude_limited,
            exclude_unique=exclude_unique,
            sort_by_price=sort_by_price,
            offset=offset,
            limit=limit,
        )
        return OwnedGifts._parse(me=self, d=result["result"])



import tgram


class DeleteStory:
    async def delete_story(
        self: "tgram.TgBot",
        business_connection_id: str,
        story_id: int,
    ) -> bool:
        """
        Deletes a story previously posted by the bot on behalf of a managed business account.
        Requires the can_manage_stories business bot right.

        Telegram documentation: https://core.telegram.org/bots/api#deletestory

        :param business_connection_id: Unique identifier of the business connection
        :type business_connection_id: :obj:`str`

        :param story_id: Unique identifier of the story to delete
        :type story_id: :obj:`int`

        :return: True on success.
        :rtype: :obj:`bool`
        """
        result = await self(
            "deleteStory",
            business_connection_id=business_connection_id,
            story_id=story_id,
        )
        return result["result"]



import tgram
from .type_ import Type_

from typing import List, Optional


class StickerSet(Type_):
    """
    This object represents a sticker set.

    Telegram Documentation: https://core.telegram.org/bots/api#stickerset

    :param name: Sticker set name
    :type name: :obj:`str`

    :param title: Sticker set title
    :type title: :obj:`str`

    :param sticker_type: Type of stickers in the set, currently one of “regular”, “mask”, “custom_emoji”
    :type sticker_type: :obj:`str`

    :param stickers: List of all set stickers
    :type stickers: :obj:`list` of :class:`tgram.types.Sticker`

    :param thumbnail: Optional. Sticker set thumbnail in the .WEBP, .TGS, or .WEBM format
    :type thumbnail: :class:`tgram.types.PhotoSize`

    :return: Instance of the class
    :rtype: :class:`tgram.types.StickerSet`
    """

    def __init__(
        self,
        name: "str" = None,
        title: "str" = None,
        sticker_type: "str" = None,
        stickers: List["tgram.types.Sticker"] = None,
        thumbnail: "tgram.types.PhotoSize" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.name = name
        self.title = title
        self.sticker_type = sticker_type
        self.stickers = stickers
        self.thumbnail = thumbnail

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.StickerSet"]:
        return (
            StickerSet(
                me=me,
                json=d,
                name=d.get("name"),
                title=d.get("title"),
                sticker_type=d.get("sticker_type"),
                stickers=[
                    tgram.types.Sticker._parse(me=me, d=i) for i in d.get("stickers")
                ]
                if d.get("stickers")
                else None,
                thumbnail=tgram.types.PhotoSize._parse(me=me, d=d.get("thumbnail")),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram
from .type_ import Type_

from typing import List, Optional


class InputTextMessageContent(Type_):
    """
    Represents the content of a text message to be sent as the result of an inline query.

    Telegram Documentation: https://core.telegram.org/bots/api#inputtextmessagecontent

    :param message_text: Text of the message to be sent, 1-4096 characters
    :type message_text: :obj:`str`

    :param parse_mode: Optional. Mode for parsing entities in the message text. See formatting options for more
        details.
    :type parse_mode: :obj:`str`

    :param entities: Optional. List of special entities that appear in message text, which can be specified instead of
        parse_mode
    :type entities: :obj:`list` of :class:`tgram.types.MessageEntity`

    :param disable_web_page_preview: deprecated
    :type disable_web_page_preview: :obj:`bool`

    :param link_preview_options: Optional. Link preview generation options for the message
    :type link_preview_options: :class:`tgram.types.LinkPreviewOptions`

    :return: Instance of the class
    :rtype: :class:`tgram.types.InputTextMessageContent`
    """

    def __init__(
        self,
        message_text: "str" = None,
        parse_mode: "str" = None,
        entities: List["tgram.types.MessageEntity"] = None,
        link_preview_options: "tgram.types.LinkPreviewOptions" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.message_text = message_text
        self.parse_mode = parse_mode
        self.entities = entities
        self.link_preview_options = link_preview_options

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.InputTextMessageContent"]:
        return (
            InputTextMessageContent(
                me=me,
                json=d,
                message_text=d.get("message_text"),
                parse_mode=d.get("parse_mode"),
                entities=[
                    tgram.types.MessageEntity._parse(me=me, d=i)
                    for i in d.get("entities")
                ]
                if d.get("entities")
                else None,
                link_preview_options=tgram.types.LinkPreviewOptions._parse(
                    me=me, d=d.get("link_preview_options")
                ),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



from typing import Callable, Any, Union, List, Pattern, Coroutine
import tgram
import re
import inspect
import asyncio


class Filter:
    def __init__(self, filter_: Union[Callable, Coroutine]) -> None:
        self._filter = filter_

    async def __call__(self, bot: "tgram.TgBot", update: Any) -> bool:
        if inspect.iscoroutinefunction(self._filter):
            result = await self._filter(bot, update)
        else:
            loop = asyncio.get_event_loop()
            result = await loop.run_in_executor(None, self._filter, bot, update)
        return bool(result)

    def __invert__(self) -> "Filter":
        async def func(*args):
            return not await self(*args)

        return Filter(func)

    def __and__(self, other: "Filter") -> "Filter":
        async def func(*args):
            return await self(*args) and await other(*args)

        return Filter(func)

    def __or__(self, other: "Filter") -> "Filter":
        async def func(*args):
            return await self(*args) or await other(*args)

        return Filter(func)


all = Filter(lambda _, __: True)
threaded = Filter(lambda _, m: bool(getattr(m, "message_thread_id", False)))
from_user = Filter(lambda _, m: bool(getattr(m, "from_user", False)))
sender_chat = Filter(lambda _, m: bool(getattr(m, "sender_chat", False)))
business_connection_id = Filter(
    lambda _, m: bool(getattr(m, "business_connection_id", False))
)
forward = Filter(lambda _, m: bool(getattr(m, "forward_origin", False)))
topic_message = Filter(lambda _, m: bool(getattr(m, "is_topic_message", False)))
automatic_forward = Filter(lambda _, m: bool(getattr(m, "is_automatic_forward", False)))
reply = Filter(lambda _, m: bool(getattr(m, "reply_to_message", False)))
quote = Filter(lambda _, m: bool(getattr(m, "quote", False)))
reply_to_story = Filter(lambda _, m: bool(getattr(m, "reply_to_story", False)))
via_bot = Filter(lambda _, m: bool(getattr(m, "via_bot", False)))
protected_content = Filter(
    lambda _, m: bool(getattr(m, "has_protected_content", False))
)
from_offline = Filter(lambda _, m: bool(getattr(m, "is_from_offline", False)))
media_group = Filter(lambda _, m: bool(getattr(m, "media_group_id", False)))
text = Filter(lambda _, m: bool(getattr(m, "text", False)))
entities = Filter(lambda _, m: bool(getattr(m, "entities", False)))
effected_message = Filter(lambda _, m: bool(getattr(m, "effect_id", False)))
animation = Filter(lambda _, m: bool(getattr(m, "animation", False)))
audio = Filter(lambda _, m: bool(getattr(m, "audio", False)))
document = Filter(lambda _, m: bool(getattr(m, "document", False)))
photo = Filter(lambda _, m: bool(getattr(m, "photo", False)))
sticker = Filter(lambda _, m: bool(getattr(m, "sticker", False)))
story = Filter(lambda _, m: bool(getattr(m, "story", False)))
video = Filter(lambda _, m: bool(getattr(m, "video", False)))
video_note = Filter(lambda _, m: bool(getattr(m, "video_note", False)))
voice = Filter(lambda _, m: bool(getattr(m, "voice", False)))
caption = Filter(lambda _, m: bool(getattr(m, "caption", False)))
media_spoiler = Filter(lambda _, m: bool(getattr(m, "has_media_spoiler", False)))
checklist = Filter(lambda _, m: bool(getattr(m, "checklist", False)))
contact = Filter(lambda _, m: bool(getattr(m, "contact", False)))
dice = Filter(lambda _, m: bool(getattr(m, "dice", False)))
game = Filter(lambda _, m: bool(getattr(m, "game", False)))
poll = Filter(lambda _, m: bool(getattr(m, "poll", False)))
venue = Filter(lambda _, m: bool(getattr(m, "venue", False)))
location = Filter(lambda _, m: bool(getattr(m, "location", False)))
new_chat_members = Filter(lambda _, m: bool(getattr(m, "new_chat_members", False)))
left_chat_member = Filter(lambda _, m: bool(getattr(m, "left_chat_member", False)))
new_chat_title = Filter(lambda _, m: bool(getattr(m, "new_chat_title", False)))
new_chat_photo = Filter(lambda _, m: bool(getattr(m, "new_chat_photo", False)))
delete_chat_photo = Filter(lambda _, m: bool(getattr(m, "delete_chat_photo", False)))
group_chat_created = Filter(lambda _, m: bool(getattr(m, "group_chat_created", False)))
supergroup_chat_created = Filter(
    lambda _, m: bool(getattr(m, "supergroup_chat_created", False))
)
channel_chat_created = Filter(
    lambda _, m: bool(getattr(m, "channel_chat_created", False))
)
message_auto_delete_timer_changed = Filter(
    lambda _, m: bool(getattr(m, "message_auto_delete_timer_changed", False))
)
migrate_to_chat_id = Filter(lambda _, m: bool(getattr(m, "migrate_to_chat_id", False)))
migrate_from_chat_id = Filter(
    lambda _, m: bool(getattr(m, "migrate_from_chat_id", False))
)
pinned_message = Filter(lambda _, m: bool(getattr(m, "pinned_message", False)))
invoice = Filter(lambda _, m: bool(getattr(m, "invoice", False)))
successful_payment = Filter(lambda _, m: bool(getattr(m, "successful_payment", False)))
refunded_payment = Filter(lambda _, m: bool(getattr(m, "refunded_payment", False)))
users_shared = Filter(lambda _, m: bool(getattr(m, "users_shared", False)))
chat_shared = Filter(lambda _, m: bool(getattr(m, "chat_shared", False)))
connected_website = Filter(lambda _, m: bool(getattr(m, "connected_website", False)))
write_access_allowed = Filter(
    lambda _, m: bool(getattr(m, "write_access_allowed", False))
)
passport_data = Filter(lambda _, m: bool(getattr(m, "passport_data", False)))
proximity_alert_triggered = Filter(
    lambda _, m: bool(getattr(m, "proximity_alert_triggered", False))
)
boost_added = Filter(lambda _, m: bool(getattr(m, "boost_added", False)))
chat_background_set = Filter(
    lambda _, m: bool(getattr(m, "chat_background_set", False))
)
checklist_tasks_done = Filter(
    lambda _, m: bool(getattr(m, "checklist_tasks_done", False))
)
checklist_tasks_added = Filter(
    lambda _, m: bool(getattr(m, "checklist_tasks_added", False))
)
direct_message_price_changed = Filter(
    lambda _, m: bool(getattr(m, "direct_message_price_changed", False))
)
forum_topic_created = Filter(
    lambda _, m: bool(getattr(m, "forum_topic_created", False))
)
forum_topic_edited = Filter(lambda _, m: bool(getattr(m, "forum_topic_edited", False)))
forum_topic_closed = Filter(lambda _, m: bool(getattr(m, "forum_topic_closed", False)))
forum_topic_reopened = Filter(
    lambda _, m: bool(getattr(m, "forum_topic_reopened", False))
)
general_forum_topic_hidden = Filter(
    lambda _, m: bool(getattr(m, "general_forum_topic_hidden", False))
)
general_forum_topic_unhidden = Filter(
    lambda _, m: bool(getattr(m, "general_forum_topic_unhidden", False))
)
giveaway_created = Filter(lambda _, m: bool(getattr(m, "giveaway_created", False)))
giveaway = Filter(lambda _, m: bool(getattr(m, "giveaway", False)))
giveaway_winners = Filter(lambda _, m: bool(getattr(m, "giveaway_winners", False)))
giveaway_completed = Filter(lambda _, m: bool(getattr(m, "giveaway_completed", False)))
video_chat_scheduled = Filter(
    lambda _, m: bool(getattr(m, "video_chat_scheduled", False))
)
video_chat_started = Filter(lambda _, m: bool(getattr(m, "video_chat_started", False)))
video_chat_ended = Filter(lambda _, m: bool(getattr(m, "video_chat_ended", False)))
video_chat_participants_invited = Filter(
    lambda _, m: bool(getattr(m, "video_chat_participants_invited", False))
)
web_app_data = Filter(lambda _, m: bool(getattr(m, "web_app_data", False)))
reply_markup = Filter(lambda _, m: bool(getattr(m, "reply_markup", False)))

service = Filter(lambda _, m: isinstance(m, tgram.types.Message) and m.service)
media = Filter(lambda _, m: isinstance(m, tgram.types.Message) and m.media)


def sender(ids: Union[str, int, List[Union[str, int]]]) -> Filter:
    """Filter messages coming from one or more sender chat"""
    ids = (
        {ids.lower() if isinstance(ids, str) else ids}
        if not isinstance(ids, list)
        else {i.lower() if isinstance(i, str) else i for i in ids}
    )
    return Filter(
        lambda _, m: getattr(m, "sender_chat")
        and (
            m.sender_chat.id in ids
            or (m.sender_chat.username and m.sender_chat.username.lower() in ids)
        )
    )


def user(ids: Union[str, int, List[Union[str, int]]]) -> Filter:
    """Filter messages coming from one or more users"""
    ids = (
        {ids.lower() if isinstance(ids, str) else ids}
        if not isinstance(ids, list)
        else {i.lower() if isinstance(i, str) else i for i in ids}
    )
    return Filter(
        lambda _, m: getattr(m, "from_user")
        and (
            m.from_user.id in ids
            or (m.from_user.username and m.from_user.username.lower() in ids)
        )
    )


def chat(ids: Union[str, int, List[Union[str, int]]]) -> Filter:
    """Filter messages coming from one or more chats"""
    ids = (
        {ids.lower() if isinstance(ids, str) else ids}
        if not isinstance(ids, list)
        else {i.lower() if isinstance(i, str) else i for i in ids}
    )
    return Filter(
        lambda _, m: getattr(m, "chat")
        and (m.chat.id in ids or (m.chat.username and m.chat.username.lower() in ids))
    )


def regex(pattern: Union[str, Pattern], flags: int = 0) -> Filter:
    """Filter updates that match a given regular expression pattern."""
    compiler = pattern if isinstance(pattern, Pattern) else re.compile(pattern, flags)

    async def regex_filter(_, m):
        if not isinstance(
            m,
            (
                tgram.types.Message,
                tgram.types.CallbackQuery,
                tgram.types.InlineQuery,
                tgram.types.PreCheckoutQuery,
            ),
        ):
            raise ValueError(f"Regex filter doesn't work with {m.__class__.__name__}")

        value = (
            (m.text or m.caption)
            if isinstance(m, tgram.types.Message)
            else m.data
            if isinstance(m, tgram.types.CallbackQuery)
            else m.query
            if isinstance(m, tgram.types.InlineQuery)
            else m.invoice_payload
            if isinstance(m, tgram.types.PreCheckoutQuery)
            else None
        )

        if value is None:
            return False

        m.matches = list(compiler.finditer(value)) or None
        return bool(m.matches)

    return Filter(regex_filter)


def chat_type(types: Union[list, str]) -> Filter:
    """Filter updates that match a given chat type."""
    types = (
        {types.lower()} if not isinstance(types, list) else {i.lower() for i in types}
    )

    async def chat_filter(_, m):
        if isinstance(m, tgram.types.CallbackQuery) and m.message and m.message.chat:
            chat_type = m.message.chat.type
        elif isinstance(m, tgram.types.InlineQuery):
            chat_type = m.chat_type
        elif getattr(m, "chat"):  # Most of other updates types have chat attribute.
            chat_type = m.chat.type
        else:
            raise ValueError(
                f"Chat type filter doesn't work with {m.__class__.__name__}"
            )

        return bool(chat_type in types)

    return Filter(chat_filter)


private = chat_type("private")
group = chat_type(["group", "supergroup"])


def command(
    commands: Union[str, List[str]],
    prefixes: Union[str, List[str]] = "/",
    case_sensitive: bool = False,
) -> Filter:
    """Filter commands, i.e.: text messages starting with "/" or any other custom prefix."""
    commands = commands if isinstance(commands, list) else [commands]
    commands = {c if case_sensitive else c.lower() for c in commands}

    prefixes = [] if prefixes is None else prefixes
    prefixes = prefixes if isinstance(prefixes, list) else [prefixes]
    prefixes = set(prefixes) if prefixes else {""}
    command_re = re.compile(r"([\"'])(.*?)(?<!\\)\1|(\S+)")

    async def filter_func(b: "tgram.TgBot", m: "tgram.types.Message"):
        text = m.text or m.caption
        username = b.me.username

        if text is None:
            return False

        for prefix in prefixes:
            if not text.startswith(prefix):
                continue

            without_prefix = text[len(prefix) :]

            for cmd in commands:
                if not re.match(
                    rf"^(?:{cmd}(?:@?{username})?)(?:\s|$)",
                    without_prefix,
                    flags=re.IGNORECASE if not case_sensitive else 0,
                ):
                    continue

                without_command = re.sub(
                    rf"{cmd}(?:@?{username})?\s?",
                    "",
                    without_prefix,
                    count=1,
                    flags=re.IGNORECASE if not case_sensitive else 0,
                )
                m.command = [cmd] + [
                    re.sub(r"\\([\"'])", r"\1", m.group(2) or m.group(3) or "")
                    for m in command_re.finditer(without_command)
                ]
                return True

        return False

    return Filter(filter_func)



import tgram


class Close:
    async def close(self: "tgram.TgBot") -> bool:
        """
        Use this method to close the bot instance before moving it from one local server to another.
        You need to delete the webhook before calling this method to ensure that the bot isn't launched again
        after server restart.
        The method will return error 429 in the first 10 minutes after the bot is launched.
        Returns True on success.

        Telegram documentation: https://core.telegram.org/bots/api#close

        :return: :obj:`bool`
        """

        result = await self(
            "close",
        )
        return result["result"]



import tgram
from tgram.types import InlineKeyboardMarkup
from tgram.types import Message
from tgram.types import ReplyParameters

from tgram.utils import convert_to_inline_keyboard_markup


class SendGame:
    async def send_game(
        self: "tgram.TgBot",
        chat_id: int,
        game_short_name: str,
        business_connection_id: str = None,
        message_thread_id: int = None,
        disable_notification: bool = None,
        protect_content: bool = None,
        message_effect_id: str = None,
        reply_parameters: ReplyParameters = None,
        reply_markup: InlineKeyboardMarkup = None,
        allow_paid_broadcast: bool = None,
    ) -> Message:
        """
        Used to send the game.

        Telegram documentation: https://core.telegram.org/bots/api#sendgame

        :param chat_id: Unique identifier for the target chat or username of the target channel (in the format @channelusername)
        :type chat_id: :obj:`int` or :obj:`str`

        :param game_short_name: Short name of the game, serves as the unique identifier for the game. Set up your games via @BotFather.
        :type game_short_name: :obj:`str`

        :param disable_notification: Sends the message silently. Users will receive a notification with no sound.
        :type disable_notification: :obj:`bool`

        :param reply_to_message_id: Deprecated - Use reply_parameters instead. If the message is a reply, ID of the original message
        :type reply_to_message_id: :obj:`int`

        :param reply_markup: Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove reply keyboard or to force a reply from the user.
        :type reply_markup: :obj:`InlineKeyboardMarkup` or :obj:`ReplyKeyboardMarkup` or :obj:`ReplyKeyboardRemove` or :obj:`ForceReply`

        :param timeout: Timeout in seconds for waiting for a response from the bot.
        :type timeout: :obj:`int`

        :param allow_sending_without_reply: Deprecated - Use reply_parameters instead. Pass True, if the message should be sent even if one of the specified replied-to messages is not found.
        :type allow_sending_without_reply: :obj:`bool`

        :param protect_content: Pass True, if content of the message needs to be protected from being viewed by the bot.
        :type protect_content: :obj:`bool`

        :param message_thread_id: Identifier of the thread to which the message will be sent.
        :type message_thread_id: :obj:`int`

        :param reply_parameters: Reply parameters.
        :type reply_parameters: :class:`tgram.types.ReplyParameters`

        :param business_connection_id: Identifier of the business connection.
        :type business_connection_id: :obj:`str`

        :param message_effect_id: Identifier of the message effect.
        :type message_effect_id: :obj:`str`

        :param allow_paid_broadcast: Pass True to allow up to 1000 messages per second, ignoring broadcasting limits for a fee of 0.1 Telegram Stars per message.
            The relevant Stars will be withdrawn from the bot's balance
        :type allow_paid_broadcast: :obj:`bool`

        :return: On success, the sent Message is returned.
        :rtype: :obj:`tgram.types.Message`
        """

        result = await self(
            "sendGame",
            chat_id=chat_id,
            game_short_name=game_short_name,
            business_connection_id=business_connection_id,
            message_thread_id=message_thread_id,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self.protect_content,
            message_effect_id=message_effect_id,
            reply_parameters=reply_parameters,
            reply_markup=convert_to_inline_keyboard_markup(reply_markup)
            if isinstance(reply_markup, list)
            else reply_markup,
            allow_paid_broadcast=allow_paid_broadcast,
        )
        return Message._parse(me=self, d=result["result"])



import tgram


class EditUserStarSubscription:
    async def edit_user_star_subscription(
        self: "tgram.TgBot",
        user_id: int,
        telegram_payment_charge_id: str,
        is_canceled: bool,
    ) -> bool:
        """
        Allows the bot to cancel or re-enable extension of a subscription paid in Telegram Stars. Returns True on success.
        """
        result = await self(
            "editUserStarSubscription",
            user_id=user_id,
            telegram_payment_charge_id=telegram_payment_charge_id,
            is_canceled=is_canceled,
        )
        return result["result"]



import tgram
from tgram.types import InputSticker


class ReplaceStickerInSet:
    async def replace_sticker_in_set(
        self: "tgram.TgBot",
        user_id: int,
        name: str,
        old_sticker: str,
        sticker: InputSticker,
    ) -> bool:
        """
        Use this method to replace an existing sticker in a sticker set with a new one. The method is equivalent to calling deleteStickerFromSet, then addStickerToSet,
            then setStickerPositionInSet. Returns True on success.

        Telegram documentation: https://core.telegram.org/bots/api#replaceStickerInSet

        :param user_id: User identifier of the sticker set owner
        :type user_id: :obj:`int`

        :param name: Sticker set name
        :type name: :obj:`str`

        :param old_sticker: File identifier of the replaced sticker
        :type old_sticker: :obj:`str`

        :param sticker: A JSON-serialized object with information about the added sticker. If exactly the same sticker had already been added to the set, then the set remains unchanged.
        :type sticker: :class:`tgram.types.InputSticker`

        :return: Returns True on success.
        :rtype: :obj:`bool`
        """

        result = await self(
            "replaceStickerInSet",
            user_id=user_id,
            name=name,
            old_sticker=old_sticker,
            sticker=sticker,
        )
        return result["result"]



import tgram
from typing import List
from typing import Union
from tgram.types import ForceReply
from tgram.types import InlineKeyboardMarkup
from tgram.types import Message
from tgram.types import MessageEntity
from tgram.types import ReplyKeyboardMarkup
from tgram.types import ReplyKeyboardRemove
from tgram.types import ReplyParameters, ParseMode
from pathlib import Path
from tgram.utils import get_file_path, get_parse_mode, convert_to_inline_keyboard_markup


class SendDocument:
    async def send_document(
        self: "tgram.TgBot",
        chat_id: Union[int, str],
        document: Union[Path, bytes, str],
        business_connection_id: str = None,
        message_thread_id: int = None,
        thumbnail: Union[Path, bytes, str] = None,
        caption: str = None,
        parse_mode: ParseMode = None,
        caption_entities: List[MessageEntity] = None,
        disable_content_type_detection: bool = None,
        disable_notification: bool = None,
        protect_content: bool = None,
        message_effect_id: str = None,
        reply_parameters: ReplyParameters = None,
        reply_markup: Union[
            InlineKeyboardMarkup, ReplyKeyboardMarkup, ReplyKeyboardRemove, ForceReply
        ] = None,
        allow_paid_broadcast: bool = None,
    ) -> Message:
        """
        Use this method to send general files.

        Telegram documentation: https://core.telegram.org/bots/api#senddocument

        :param chat_id: Unique identifier for the target chat or username of the target channel (in the format @channelusername)
        :type chat_id: :obj:`int` or :obj:`str`

        :param document: (document) File to send. Pass a file_id as String to send a file that exists on the Telegram servers (recommended), pass an HTTP URL as a
            String for Telegram to get a file from the Internet, or upload a new one using multipart/form-data
        :type document: :obj:`str` or :class:`tgram.types.InputFile`

        :param reply_to_message_id: Deprecated - Use reply_parameters instead. If the message is a reply, ID of the original message
        :type reply_to_message_id: :obj:`int`

        :param caption: Document caption (may also be used when resending documents by file_id), 0-1024 characters after entities parsing
        :type caption: :obj:`str`

        :param reply_markup: Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove reply keyboard
            or to force a reply from the user.
        :type reply_markup: :class:`tgram.types.InlineKeyboardMarkup` or :class:`tgram.types.ReplyKeyboardMarkup` or :class:`tgram.types.ReplyKeyboardRemove`
            or :class:`tgram.types.ForceReply`

        :param parse_mode: Mode for parsing entities in the document caption
        :type parse_mode: :obj:`str`

        :param disable_notification: Sends the message silently. Users will receive a notification with no sound.
        :type disable_notification: :obj:`bool`

        :param timeout: Timeout in seconds for the request.
        :type timeout: :obj:`int`

        :param thumbnail: InputFile or String : Thumbnail of the file sent; can be ignored if thumbnail generation for the file is supported server-side. The thumbnail should be in JPEG format and less than 200 kB in size. A thumbnail's width and height should not exceed 320. Ignored if the file is not uploaded using multipart/form-data. Thumbnails can't be reused and can be only uploaded as a new file, so you can pass “attach://<file_attach_name>” if the thumbnail was uploaded using multipart/form-data under <file_attach_name>
        :type thumbnail: :obj:`str` or :class:`tgram.types.InputFile`

        :param caption_entities: A JSON-serialized list of special entities that appear in the caption, which can be specified instead of parse_mode
        :type caption_entities: :obj:`list` of :class:`tgram.types.MessageEntity`

        :param allow_sending_without_reply: Deprecated - Use reply_parameters instead. Pass True, if the message should be sent even if the specified replied-to message is not found
        :type allow_sending_without_reply: :obj:`bool`

        :param visible_file_name: allows to define file name that will be visible in the Telegram instead of original file name
        :type visible_file_name: :obj:`str`

        :param disable_content_type_detection: Disables automatic server-side content type detection for files uploaded using multipart/form-data
        :type disable_content_type_detection: :obj:`bool`

        :param data: function typo miss compatibility: do not use it
        :type data: :obj:`str`

        :param protect_content: Protects the contents of the sent message from forwarding and saving
        :type protect_content: :obj:`bool`

        :param message_thread_id: Identifier of a message thread, in which the message will be sent
        :type message_thread_id: :obj:`int`

        :param thumb: Deprecated. Use thumbnail instead
        :type thumb: :obj:`str` or :class:`tgram.types.InputFile`

        :param reply_parameters: Reply parameters.
        :type reply_parameters: :class:`tgram.types.ReplyParameters`

        :param business_connection_id: Unique identifier for the target business connection
        :type business_connection_id: :obj:`str`

        :param message_effect_id: Unique identifier for the message effect
        :type message_effect_id: :obj:`str`

        :param allow_paid_broadcast: Pass True to allow up to 1000 messages per second, ignoring broadcasting limits for a fee of 0.1 Telegram Stars per message.
            The relevant Stars will be withdrawn from the bot's balance
        :type allow_paid_broadcast: :obj:`bool`

        :return: On success, the sent Message is returned.
        :rtype: :class:`tgram.types.Message`
        """

        result = await self(
            "sendDocument",
            chat_id=chat_id,
            document=get_file_path(document),
            business_connection_id=business_connection_id,
            message_thread_id=message_thread_id,
            thumbnail=get_file_path(thumbnail),
            caption=caption,
            parse_mode=get_parse_mode(self, parse_mode),
            caption_entities=caption_entities,
            disable_content_type_detection=disable_content_type_detection,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self.protect_content,
            message_effect_id=message_effect_id,
            reply_parameters=reply_parameters,
            reply_markup=convert_to_inline_keyboard_markup(reply_markup)
            if isinstance(reply_markup, list)
            else reply_markup,
            allow_paid_broadcast=allow_paid_broadcast,
        )
        return Message._parse(me=self, d=result["result"])



import os

from io import BytesIO
from pathlib import Path


def get_file_name(obj):
    name = getattr(obj, "name", None)
    if name and isinstance(name, str) and name[0] != "<" and name[-1] != ">":
        return os.path.basename(name)


def get_file_path(file):
    if isinstance(file, BytesIO):
        file.seek(0)
    return Path(file) if isinstance(file, str) and os.path.isfile(file) else file



import tgram
from typing import List
from typing import Union
from tgram.types import ForceReply
from tgram.types import InlineKeyboardMarkup
from tgram.types import Message
from tgram.types import MessageEntity
from tgram.types import ReplyKeyboardMarkup
from tgram.types import ReplyKeyboardRemove
from tgram.types import ReplyParameters, ParseMode
from pathlib import Path
from tgram.utils import get_file_path, get_parse_mode, convert_to_inline_keyboard_markup


class SendPhoto:
    async def send_photo(
        self: "tgram.TgBot",
        chat_id: Union[int, str],
        photo: Union[Path, bytes, str],
        business_connection_id: str = None,
        message_thread_id: int = None,
        caption: str = None,
        parse_mode: ParseMode = None,
        caption_entities: List[MessageEntity] = None,
        show_caption_above_media: bool = None,
        has_spoiler: bool = None,
        disable_notification: bool = None,
        protect_content: bool = None,
        message_effect_id: str = None,
        reply_parameters: ReplyParameters = None,
        reply_markup: Union[
            InlineKeyboardMarkup, ReplyKeyboardMarkup, ReplyKeyboardRemove, ForceReply
        ] = None,
        allow_paid_broadcast: bool = None,
    ) -> Message:
        """
        Use this method to send photos. On success, the sent Message is returned.

        Telegram documentation: https://core.telegram.org/bots/api#sendphoto

        :param chat_id: Unique identifier for the target chat or username of the target channel (in the format @channelusername)
        :type chat_id: :obj:`int` or :obj:`str`

        :param photo: Photo to send. Pass a file_id as String to send a photo that exists on the Telegram servers (recommended),
            pass an HTTP URL as a String for Telegram to get a photo from the Internet, or upload a new photo using multipart/form-data.
            The photo must be at most 10 MB in size. The photo's width and height must not exceed 10000 in total. Width and height ratio must be at most 20.
        :type photo: :obj:`str` or :class:`tgram.types.InputFile`

        :param caption: Photo caption (may also be used when resending photos by file_id), 0-1024 characters after entities parsing
        :type caption: :obj:`str`

        :param parse_mode: Mode for parsing entities in the photo caption.
        :type parse_mode: :obj:`str`

        :param caption_entities: A JSON-serialized list of special entities that appear in the caption, which can be specified instead of parse_mode
        :type caption_entities: :obj:`list` of :class:`tgram.types.MessageEntity`

        :param disable_notification: Sends the message silently. Users will receive a notification with no sound.
        :type disable_notification: :obj:`bool`

        :param protect_content: Protects the contents of the sent message from forwarding and saving
        :type protect_content: :obj:`bool`

        :param reply_to_message_id: Deprecated - Use reply_parameters instead. If the message is a reply, ID of the original message
        :type reply_to_message_id: :obj:`int`

        :param allow_sending_without_reply: Deprecated - Use reply_parameters instead. Pass True, if the message should be sent even if the specified replied-to message is not found
        :type allow_sending_without_reply: :obj:`bool`

        :param reply_markup: Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions
            to remove reply keyboard or to force a reply from the user.
        :type reply_markup: :class:`tgram.types.InlineKeyboardMarkup` or :class:`tgram.types.ReplyKeyboardMarkup` or :class:`tgram.types.ReplyKeyboardRemove`
            or :class:`tgram.types.ForceReply`

        :param timeout: Timeout in seconds for the request.
        :type timeout: :obj:`int`

        :param message_thread_id: Identifier of a message thread, in which the message will be sent
        :type message_thread_id: :obj:`int`

        :param has_spoiler: Pass True, if the photo should be sent as a spoiler
        :type has_spoiler: :obj:`bool`

        :param reply_parameters: Reply parameters.
        :type reply_parameters: :class:`tgram.types.ReplyParameters`

        :param business_connection_id: Unique identifier for the target business connection
        :type business_connection_id: :obj:`str`

        :param message_effect_id: Unique identifier for the message effect
        :type message_effect_id: :obj:`str`

        :param show_caption_above_media: Pass True, if the caption must be shown above the message media. Supported only for animation, photo and video messages.
        :type show_caption_above_media: :obj:`bool`

        :param allow_paid_broadcast: Pass True to allow up to 1000 messages per second, ignoring broadcasting limits for a fee of 0.1 Telegram Stars per message.
            The relevant Stars will be withdrawn from the bot's balance
        :type allow_paid_broadcast: :obj:`bool`

        :return: On success, the sent Message is returned.
        :rtype: :class:`tgram.types.Message`
        """

        result = await self(
            "sendPhoto",
            chat_id=chat_id,
            photo=get_file_path(photo),
            business_connection_id=business_connection_id,
            message_thread_id=message_thread_id,
            caption=caption,
            parse_mode=get_parse_mode(self, parse_mode),
            caption_entities=caption_entities,
            show_caption_above_media=show_caption_above_media,
            has_spoiler=has_spoiler,
            disable_notification=disable_notification,
            protect_content=protect_content
            if protect_content is not None
            else self.protect_content,
            message_effect_id=message_effect_id,
            reply_parameters=reply_parameters,
            reply_markup=convert_to_inline_keyboard_markup(reply_markup)
            if isinstance(reply_markup, list)
            else reply_markup,
            allow_paid_broadcast=allow_paid_broadcast,
        )
        return Message._parse(me=self, d=result["result"])



import tgram
import random

from typing import List, Optional


class InputChecklistTask:
    """
    Describes a task to add to a checklist.

    :param id: Unique identifier of the task; must be positive and unique among all task identifiers currently present in the checklist
    :type id: int

    :param text: Text of the task; 1-100 characters after entities parsing
    :type text: str

    :param parse_mode: Optional. Mode for parsing entities in the text. See formatting options for more details.
    :type parse_mode: str

    :param text_entities: Optional. List of special entities that appear in the text, which can be specified instead of parse_mode.
        Currently, only bold, italic, underline, strikethrough, spoiler, and custom_emoji entities are allowed.
    :type text_entities: List[tgram.types.MessageEntity]
    """

    def __init__(
        self,
        text: str,
        parse_mode: Optional[str] = None,
        text_entities: Optional[List["tgram.types.MessageEntity"]] = None,
    ):
        self.id = random.randint(10000, 99999)
        self.text = text
        self.parse_mode = parse_mode
        self.text_entities = text_entities

    @staticmethod
    def _parse(d: dict = None) -> Optional["InputChecklistTask"]:
        return (
            InputChecklistTask(
                id=d.get("id"),
                text=d.get("text"),
                parse_mode=d.get("parse_mode"),
                text_entities=[
                    tgram.types.MessageEntity._parse(d=i)
                    for i in d.get("text_entities")
                ]
                if d.get("text_entities")
                else None,
            )
            if d
            else None
        )



import tgram
from .type_ import Type_
from typing import Optional


class StarAmount(Type_):
    """
    Describes an amount of Telegram Stars.

    Telegram documentation: https://core.telegram.org/bots/api#staramount

    :param amount: Integer amount of Telegram Stars, rounded to 0; can be negative
    :type amount: :obj:`int`

    :param nanostar_amount: Optional. The number of 1/1000000000 shares of Telegram Stars; from -999999999 to 999999999; can be negative if and only if amount is non-positive
    :type nanostar_amount: :obj:`int`

    :return: Instance of the class
    :rtype: :class:`StarAmount`
    """

    def __init__(
        self,
        amount: "int" = None,
        nanostar_amount: "int" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.amount = amount
        self.nanostar_amount = nanostar_amount

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["StarAmount"]:
        return (
            StarAmount(
                me=me,
                json=d,
                amount=d.get("amount"),
                nanostar_amount=d.get("nanostar_amount"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram
from .type_ import Type_

from typing import Optional


class WebAppData(Type_):
    """
    Describes data sent from a Web App to the bot.

    Telegram Documentation: https://core.telegram.org/bots/api#webappdata

    :param data: The data. Be aware that a bad client can send arbitrary data in this field.
    :type data: :obj:`str`

    :param button_text: Text of the web_app keyboard button from which the Web App was opened. Be aware that a bad client
        can send arbitrary data in this field.
    :type button_text: :obj:`str`

    :return: Instance of the class
    :rtype: :class:`tgram.types.WebAppData`
    """

    def __init__(
        self,
        data: "str" = None,
        button_text: "str" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.data = data
        self.button_text = button_text

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.WebAppData"]:
        return (
            WebAppData(
                me=me, json=d, data=d.get("data"), button_text=d.get("button_text")
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram


class VerifyUser:
    async def verify_user(
        self: "tgram.TgBot", user_id: int, custom_description: str = None
    ) -> bool:
        """
        Verifies a user on behalf of the organization which is represented by the bot. Returns True on success.
        Telegram documentation: https://core.telegram.org/bots/api#verifyuser

        :param user_id: Unique identifier of the target user
        :type user_id: :obj:`int`

        :param custom_description: UCustom description for the verification; 0-70 characters.
            Must be empty if the organization isn't allowed to provide a custom verification description.
        :type custom_description: :obj:`str`

        :return: True on success
        :rtype: :obj:`bool`
        """

        result = await self(
            "verifyUser", user_id=user_id, custom_description=custom_description
        )
        return result["result"]



import tgram
from typing import Union


class ExportChatInviteLink:
    async def export_chat_invite_link(
        self: "tgram.TgBot", chat_id: Union[int, str]
    ) -> str:
        """
        Use this method to export an invite link to a supergroup or a channel. The bot must be an administrator
        in the chat for this to work and must have the appropriate admin rights.

        Telegram documentation: https://core.telegram.org/bots/api#exportchatinvitelink

        :param chat_id: Id: Unique identifier for the target chat or username of the target channel
            (in the format @channelusername)
        :type chat_id: :obj:`int` or :obj:`str`

        :return: exported invite link as String on success.
        :rtype: :obj:`str`
        """

        result = await self(
            "exportChatInviteLink",
            chat_id=chat_id,
        )
        return result["result"]



import tgram
from tgram.types import MenuButton


class GetChatMenuButton:
    async def get_chat_menu_button(
        self: "tgram.TgBot", chat_id: int = None
    ) -> MenuButton:
        """
        Use this method to get the current value of the bot's menu button
        in a private chat, or the default menu button.
        Returns MenuButton on success.

        Telegram Documentation: https://core.telegram.org/bots/api#getchatmenubutton

        :param chat_id: Unique identifier for the target private chat.
            If not specified, default bot's menu button will be returned.
        :type chat_id: :obj:`int` or :obj:`str`

        :return: types.MenuButton
        :rtype: :class:`tgram.types.MenuButton`
        """

        result = await self(
            "getChatMenuButton",
            chat_id=chat_id,
        )
        return MenuButton._parse(me=self, d=result["result"])



import tgram


class SetMyShortDescription:
    async def set_my_short_description(
        self: "tgram.TgBot", short_description: str = None, language_code: str = None
    ) -> bool:
        """
        Use this method to change the bot's short description, which is shown on the bot's profile page and
        is sent together with the link when users share the bot.
        Returns True on success.

        :param short_description: New short description for the bot; 0-120 characters. Pass an empty string to remove the dedicated short description for the given language.
        :type short_description: :obj:`str`

        :param language_code: A two-letter ISO 639-1 language code.
            If empty, the short description will be applied to all users for whose language there is no dedicated short description.
        :type language_code: :obj:`str`

        :return: True on success.
        """

        result = await self(
            "setMyShortDescription",
            short_description=short_description,
            language_code=language_code,
        )
        return result["result"]



import tgram
from .type_ import Type_

from typing import Optional


class MessageOriginChannel(Type_):
    """
    The message was originally sent to a channel chat.

    :param chat: Channel chat to which the message was originally sent
    :type chat: :class:`Chat`

    :param message_id: Unique message identifier inside the chat
    :type message_id: :obj:`int`

    :param author_signature: Optional. Signature of the original post author
    :type author_signature: :obj:`str`
    """

    def __init__(
        self,
        type: "str" = None,
        date: "int" = None,
        chat: "tgram.types.Chat" = None,
        message_id: "int" = None,
        author_signature: "str" = None,
        me: "tgram.TgBot" = None,
        json: "dict" = None,
    ):
        super().__init__(me=me, json=json)
        self.type = type
        self.date = date
        self.chat = chat
        self.message_id = message_id
        self.author_signature = author_signature

    @staticmethod
    def _parse(
        me: "tgram.TgBot" = None, d: dict = None, force: bool = None
    ) -> Optional["tgram.types.MessageOriginChannel"]:
        return (
            MessageOriginChannel(
                me=me,
                json=d,
                type=d.get("type"),
                date=d.get("date"),
                chat=tgram.types.Chat._parse(me=me, d=d.get("chat")),
                message_id=d.get("message_id"),
                author_signature=d.get("author_signature"),
            )
            if d and (force or me and __class__.__name__ not in me._custom_types)
            else None
            if not d
            else Type_._custom_parse(
                __class__._parse(me=me, d=d, force=True),
                me._custom_types.get(__class__.__name__),
            )
        )



import tgram
from typing import Union


class SetChatStickerSet:
    async def set_chat_sticker_set(
        self: "tgram.TgBot", chat_id: Union[int, str], sticker_set_name: str
    ) -> bool:
        """
        Use this method to set a new group sticker set for a supergroup. The bot must be an administrator in the chat
        for this to work and must have the appropriate administrator rights. Use the field can_set_sticker_set optionally returned
        in getChat requests to check if the bot can use this method. Returns True on success.

        Telegram documentation: https://core.telegram.org/bots/api#setchatstickerset

        :param chat_id: Unique identifier for the target chat or username of the target supergroup (in the format @supergroupusername)
        :type chat_id: :obj:`int` or :obj:`str`

        :param sticker_set_name: Name of the sticker set to be set as the group sticker set
        :type sticker_set_name: :obj:`str`

        :return: StickerSet object
        :rtype: :class:`tgram.types.StickerSet`
        """

        result = await self(
            "setChatStickerSet",
            chat_id=chat_id,
            sticker_set_name=sticker_set_name,
        )
        return result["result"]
